/*
    Copyright (C) 2010 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef APPLICATIONINFO_H
#define APPLICATIONINFO_H

#include<cstddef>
#include<QXmlSimpleReader>
#include<QXmlDefaultHandler>
#include<QDir>
#include<QFileInfo>
#include<QFile>
#include<QTreeWidget>

class Glob;
class ApplicationDirectory;
class ApplicationCategory;
typedef QList<Glob *> ListGlobs;
typedef QList<ApplicationDirectory *> ListDirectories;
typedef QList<ApplicationCategory *> ListCategories;

/**
  File glob (*.png for example)
  */
class Glob {
public:
    Glob(QString &_value, QString &_filter);
    ~Glob();

    /**
      Determine if this glob is for adding file to the delete file list, or to remove them.
      \return true for include, false for exclude.
      */
    bool isInclusive() const {
        return filter;
    }

    /**
      \return The glob value (like '*.png')
      */
    const QString *getValue() const {
        return value;
    }

private:
    QString *value;
    bool filter;
};

/**
  Contains the directory name and files globs associated.
  */
class ApplicationDirectory {
    friend class ApplicationInfo;

public:
    ApplicationDirectory(const QString &_path, const QString &_recursive);
    ~ApplicationDirectory();

    /**
      Search recursively.
      */
    bool isRecursive() const {
        return recursive;
    }

    bool isAvailable() const { return exists; }

    qint64 addFileToList(QTreeWidget *treeWidget) const;

    const QString &getPath() const { return *path; }

private:
    /**
     * Recursively add files to the treewidget.
     * @param treeWidget The TreeWidget parent
     * @param qDir The directory to recurse
     */
    qint64 addDirectoryToList(QTreeWidget *treeWidget, const QDir &qDir) const;

    bool recursive;
    bool exists;
    QString *path;
    QList<QDir*> *pathList;
    ListGlobs *globs;
};


/*!
  \class ApplicationCategory
  \brief This class contains elements for an sub application type (like cache directories)
  */
class ApplicationCategory {
    friend class ApplicationInfo;
public:
    ApplicationCategory();

    ~ApplicationCategory();

    const QString *getName() { return name; }
    void setName(const QString &_name) {
        name = new QString(_name);
    }

    const QString *getIcon() { return icon; }
    void setIcon(const QString &_icon) {
        icon = new QString(_icon);
    }

    Qt::CheckState getChecked() { return checked; }
    void setChecked(const Qt::CheckState _value) {
      if (_value == Qt::PartiallyChecked) {
	qWarning("A category can't be partially checked");
	checked = Qt::Checked;
      } else {
	checked = _value;
      }
    }

    ListDirectories *getDirectories() { return directories; }
    void addDirectory(ApplicationDirectory *directory);

    bool isDisplayable() const;

private:
    QString *name;
    QString *icon;
    Qt::CheckState checked;
    ListDirectories *directories;
};


/*!
  \class ApplicationInfo
  \brief Include alls elements needed for an application (eg: firefox, thumbnail ...)
 */
class ApplicationInfo : public QXmlDefaultHandler {
public:
    ApplicationInfo(const QFileInfo *inputFile);
    ~ApplicationInfo();

    const QString *getName() { return name; }
    void setName(const QString &_name);

    const QString *getIcon() { return icon; }
    void setIcon(const QString &_icon);

    Qt::CheckState getChecked() { return checked; }
    void setChecked(const Qt::CheckState _value) { checked = _value; }

    const ListCategories *getCategories() const { return categories; }

    /*!
      \return true if the file have been successfully read, and theses categories can be displayed.
      */
    bool isDisplayable() const { return this->displayable; }

    // QXmlDefaultHandler reimp:
    bool startElement (const QString &namespaceURI, const QString &localName, const QString &qName, const QXmlAttributes &attributes);
    bool endElement (const QString &namespaceURI, const QString &localName, const QString &qName);
    bool fatalError (const QXmlParseException &exception);
private:
    bool readFile(QFile *file);
    void freeCategories();

    /*!
      \brief Add an application sub type. ApplicationInfo will free the memory when freed.
      */
    void addCategory(ApplicationCategory *category);

private:
    QString *name;
    QString *icon;
    Qt::CheckState checked;
    bool displayable;
    ListCategories *categories;

};

#endif // APPLICATIONINFO_H
