/*
    Copyright (C) 2010,2011 Yvon Tanguy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<errno.h>
#include<unistd.h>
#include<iostream>
#include<string>
#include<QDebug>
#include<QApplication>
#include "sccutils.h"
#include "iconnaming.h"
#include "treewidgetapplication.h"

/********************************************/
/*         TreeWidgetApplication            */
/********************************************/

TreeWidgetApplication::TreeWidgetApplication(QTreeWidget *parent, ApplicationInfo *applicationInfo) : QTreeWidgetItem(parent, UserType) {
    this->applicationInfo = NULL;

    this->applicationInfo = applicationInfo;
    if (this->applicationInfo->isDisplayable()) {
        for (int i=0; i < this->applicationInfo->getCategories()->size(); i++) {
            new TreeWidgetCategory(this, this->applicationInfo->getCategories()->at(i));
        }

        this->setCheckState(0, this->applicationInfo->getChecked());
        this->setText(0, QApplication::translate("ApplicationName", this->applicationInfo->getName()->toStdString().c_str(), 0, QApplication::UnicodeUTF8));
        if (NULL != this->applicationInfo->getIcon() && !this->applicationInfo->getIcon()->isEmpty()) {
            this->setIcon(0, SCCUtils::getIcon(*this->applicationInfo->getIcon(), 22));
        }


    } else {
        this->setHidden(true);
        this->setCheckState(0, Qt::Unchecked);
        delete this->applicationInfo;
        this->applicationInfo = NULL;
    }

}

TreeWidgetApplication::~TreeWidgetApplication() {
//    std::cout << "TreeWidgetApplication: free memory" << std::endl;
    if (NULL != this->applicationInfo) {
        delete this->applicationInfo;
        this->applicationInfo = NULL;
    }
}

qint64 TreeWidgetApplication::initFilesWidgets(QTreeWidget *fileTW, QProgressBar *progressBar) const {
    qint64 filesSize = 0;
    if (Qt::Unchecked != this->checkState(0)) {
        for (int i=0; i < this->childCount(); i++) {
            TreeWidgetCategory *twCChild = (TreeWidgetCategory *) this->child(i);
            filesSize += twCChild->initFilesWidgets(fileTW, progressBar);
        }
    }

    return filesSize;
}

int TreeWidgetApplication::getDirectoriesCount() const {
    int count = 0;
    if (Qt::Unchecked != this->checkState(0)) {
        for (int i=0; i < this->childCount(); i++) {
            TreeWidgetCategory *twCChild = (TreeWidgetCategory *) this->child(i);
            count += twCChild->getDirectoriesCount();
        }
    }
    return count;
}

void TreeWidgetApplication::updateCheckState() {
    int numCheck = 0;
    for (int i=0; i < this->childCount(); i++) {
        TreeWidgetCategory *twCChild = (TreeWidgetCategory *) this->child(i);        
        numCheck += (twCChild->checkState(0) == Qt::Checked) ? 1 : 0;
    }

    if (0 == numCheck) {
        this->setCheckState(0, Qt::Unchecked);
    } else if (this->childCount() == numCheck) {
        this->setCheckState(0, Qt::Checked);
    } else {
        this->setCheckState(0, Qt::PartiallyChecked);
    }

}

void TreeWidgetApplication::updateChildCheckState() {
    Qt::CheckState cs = this->checkState(0);
    if (cs == Qt::PartiallyChecked)
      cs = Qt::Checked;

    for (int i=0; i < this->childCount(); i++) {
        TreeWidgetCategory *twCChild = (TreeWidgetCategory *) this->child(i);
        twCChild->setCheckState(0, cs);
    }
}

/********************************************/
/*           TreeWidgetCategory             */
/********************************************/


TreeWidgetCategory::TreeWidgetCategory(QTreeWidgetItem *parent, ApplicationCategory *category) : QTreeWidgetItem(parent, UserType) {
    this->category = category;
    if (NULL != this->category && this->category->isDisplayable()) {
        this->setCheckState(0, category->getChecked());
        this->setText(0, QApplication::translate("ApplicationName", category->getName()->toStdString().c_str(), 0, QApplication::UnicodeUTF8));
        if (NULL != category->getIcon() && !category->getIcon()->isEmpty()) {
            this->setIcon(0, SCCUtils::getIcon(*category->getIcon(), 22));
        }
    } else {
        this->setHidden(true);
        this->setCheckState(0, Qt::Unchecked);
        this->category = NULL;
    }

}

TreeWidgetCategory::~TreeWidgetCategory() {
//    std::cout << "\tTreeWidgetCategory: free memory" << std::endl;
    // deleted in TreeWidgetApplication via ApplicationInfo destructor
    this->category = NULL;

}

qint64 TreeWidgetCategory::initFilesWidgets(QTreeWidget *fileTW, QProgressBar *progressBar) const {
    qint64 filesSize = 0;
    if (Qt::Unchecked != this->checkState(0) && NULL != this->category) {
        ListDirectories *directories = this->category->getDirectories();
        for (int i=0; NULL != directories && i < directories->size(); i++) {
            ApplicationDirectory *appDirectory = directories->at(i);
            filesSize += appDirectory->addFileToList(fileTW);

            progressBar->setUpdatesEnabled(false);
            progressBar->setFormat(appDirectory->getPath() + " - %p%");
            progressBar->setValue(progressBar->value() + 1);
            progressBar->setUpdatesEnabled(true);
        }
    }
    return filesSize;
}

int TreeWidgetCategory::getDirectoriesCount() const {
    if (Qt::Unchecked != this->checkState(0) && NULL != this->category) {
        return this->category->getDirectories()->size();
    } else {
        return 0;
    }
}

/********************************************/
/*             TreeWidgetFile               */
/********************************************/

/**
 * Constructor
 * @param parent The QTreeWidget parent of this item
 * @param filename The filename of the file represent by this item.
 */
TreeWidgetFile::TreeWidgetFile(QTreeWidget *parent, QString filename) : QTreeWidgetItem(parent, UserType) {
  this->setHidden(true);
  this->fileInfo = NULL;
  
  //  if (NULL != filename) {
  this->fileInfo = new QFileInfo(filename);
  this->init();
  // }
}

/**
 * Constructor
 * @param parent The QTreeWidget parent of this item
 * @param file The file of this item. Deleted by this class/
 */
TreeWidgetFile::TreeWidgetFile(QTreeWidget *parent, QFileInfo *file) : QTreeWidgetItem(parent, UserType) {
  this->setHidden(true);
  this->fileInfo = file;

  this->init();
}

/**
 * Destructor, release the memory used by the fileInfo
 */
TreeWidgetFile::~TreeWidgetFile() {
//    std::cout << "TreeWidgetFile: free memory" << std::endl;
    if (this->fileInfo != NULL) {
        delete this->fileInfo;
        this->fileInfo = NULL;
    }

}

/**
 * Internal init. Called by constructors.
 */
void TreeWidgetFile::init() {
  if (NULL == this->fileInfo) {
    return;
  }

  if (fileInfo->exists()) {
    this->setHidden(false);
  }

  this->setText(0, fileInfo->absoluteFilePath());
  if (fileInfo->isFile()) {
    this->setText(1, SCCUtils::getHumanSize(fileInfo->size()));
    this->setIcon(0, SCCUtils::getIcon(freedesktop::IconNamingMimeTypes::TEXT_X_GENERIC, 22));
  } else if (fileInfo->isDir()) {
    this->setText(1, QObject::tr("Directory"));
    this->setIcon(0, SCCUtils::getIcon(freedesktop::IconNamingPlaces::FOLDER, 22));
  } else if (fileInfo->isSymLink()) {
    this->setText(1, QObject::tr("Symbolik link"));
    this->setIcon(0, SCCUtils::getIcon(freedesktop::IconNamingEmblems::EMBLEM_SYMBOLIC_LINK, 22));
  }
}

/**
 * Return this file size, or 0 if this file is not a regular file.
 */
qint64 TreeWidgetFile::getFileSize() const {
    if (this->fileInfo != NULL && fileInfo->isFile()) {
        return fileInfo->size();
    }
    return 0;
}

/**
 * Delete (unlink) the file or directory represented by this object instance.
 */
bool TreeWidgetFile::deleteFile() {
  if (NULL == this->fileInfo) {
    return false;
  }

  QString filePath = this->fileInfo->canonicalFilePath();
  QString msg = QObject::tr("SimpleCacheCleaner: delete %1").arg(filePath);
  const char *cStrMsg = msg.toStdString().c_str();

  if (fileInfo->isFile()) {
    if (!unlink(filePath.toStdString().c_str())) {
      return true;
    }
    perror(cStrMsg);
    
  } else if (fileInfo->isDir()) {
    if (!rmdir(filePath.toStdString().c_str())) {
      return true;
    }
    perror(cStrMsg);
  }

  return false;
}

/**
 * Return the filename.
 */
QString TreeWidgetFile::getFileName() const {
  if (this->fileInfo != NULL) {
    return this->fileInfo->fileName();
  }
  return QString();
}
