#ifndef FDEXCEPTION_H
#define FDEXCEPTION_H

#include<string>
#include<sstream>
#include<exception>
#include<iostream>
#include "libfreedesktop.h"

namespace freedesktop {

//#define FD_EXCEPTION(e) e.addTrace(__FILE__, __LINE__, __FUNCTION__)

/**
  \macro An helper macro to throw the exception.
  */
#define FD_EXCEPTION(_msg) FDException(_msg, __FILE__, __LINE__, __func__)

    /**
      \brief A simple exception used by libfreedesktop, base on the std::exception class.
      */
    class FDException : public std::exception {
    public:
        /**
          \brief Empty constructor, with a "unknown error" message.
          */
        FDException() {
            this->message = "unknown error";
        };

        /**
          \brief Exception with a message and a basic trace of the throw origin
          */
        FDException(const std::string & _message, const std::string _trace) throw() {
            message = _message;
            backTrace.push_back(_trace);
        }

        /**
          \brief Exception with a message and a full trace of the throw origin
          */
        FDException(const std::string & _message, const char *_source, int _line, const char *_function) throw() {
            message = _message;
            this->addTrace(_source, _line, _function);
        }

        ~FDException() throw() { }

        /**
          \brief Return the error message
          */
        virtual const char* what() const throw() { return message.c_str(); }

        //virtual void addTrace(std::string &_trace, int line) { backTrace.push_back(_trace); }

        /**
          \brief Add a trace in the exception, in case it is re-throw.
          */
        virtual void addTrace(const char *_source, int _line, const char *_function) {
            std::ostringstream os; //(std::ostringstream::out);

            os << _source << ":" << _line << ": " << _function << "()";

            backTrace.push_back(os.str());
        }

        /**
          \brief Return the backtrace (or stack trace) of this exception.
          \note This may be empty.
          */
        virtual const stringList &getBackTrace() const throw() { return backTrace; }

        /**
          \brief Print the backtrace to the standard error output.
          */
        virtual void printBackTrace() const throw() {
            std::cerr << "FDException: " << message << ":" << std::endl;
            for (stringList::size_type i=0; i < backTrace.size(); i++) {
                std::cerr << "\t" << backTrace.at(i) << std::endl;
            }
        }

    private:
        std::string message;
        stringList backTrace;
    };
}

#endif // FDEXCEPTION_H
