/*
    Copyright (C) 2010 Yvon Tanguy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef ICONNAMING_H
#define ICONNAMING_H

#include<string>

namespace freedesktop {

    /*
        http://standards.freedesktop.org/icon-naming-spec/icon-naming-spec-0.8.90.html

        The Icon Theme Specification has been in use now for a while, in several desktops, including KDE and Gnome.
        However, there has never been any centralized direction on how to name the icons that are available for use
        by applications, when creating a theme. This has meant that artists have historically had to duplicate
        many icons, in order for their themes to work across desktop environments.

        This specification aims to solve this problem, by laying out a standard naming scheme for icon creation,
        as well as providing a minimal list of must have icons. Addendums to this specification will provide
         additional icon lists for extended sets, such as devices and MIME types.
     */

    /*!
      \brief Icons which are generally used in menus and dialogs for interacting with the user.
      <a href="http://standards.freedesktop.org/icon-naming-spec/icon-naming-spec-0.8.90.html">icon-naming-spec</a>
      */
    class IconNamingActions {
    private:
        IconNamingActions() {}

    public:
        /*!
         \brief The icon used for the action to create a new address book.
        */
        static const std::string ADDRESS_BOOK_NEW;

        /*!
         \brief The icon used for exiting an application. Typically this is seen in the application's menus as File->Quit.
        */
        static const std::string APPLICATION_EXIT;

        /*!
         \brief The icon used for the action to create a new appointment in a calendaring application.
        */
        static const std::string APPOINTMENT_NEW;

        /*!
         \brief The icon used for initiating or accepting a call. Should be similar to the standard cellular call pickup icon, a green handset with ear and mouth pieces facing upward.
        */
        static const std::string CALL_START;

        /*!
         \brief The icon used for stopping a current call. Should be similar to the standard cellular call hangup icon, a red handset with ear and mouth pieces facing downward.
        */
        static const std::string CALL_STOP;

        /*!
         \brief The icon used for the action to create a new contact in an address book application.
        */
        static const std::string CONTACT_NEW;

        /*!
         \brief The icon used for the action to create a new document.
        */
        static const std::string DOCUMENT_NEW;

        /*!
         \brief The icon used for the action to open a document.
        */
        static const std::string DOCUMENT_OPEN;

        /*!
         \brief The icon used for the action to open a document that was recently opened.
        */
        static const std::string DOCUMENT_OPEN_RECENT;

        /*!
         \brief The icon for the page setup action of a document editor.
        */
        static const std::string DOCUMENT_PAGE_SETUP;

        /*!
         \brief The icon for the print action of an application.
        */
        static const std::string DOCUMENT_PRINT;

        /*!
         \brief The icon for the print preview action of an application.
        */
        static const std::string DOCUMENT_PRINT_PREVIEW;

        /*!
         \brief The icon for the action to view the properties of a document in an application.
        */
        static const std::string DOCUMENT_PROPERTIES;

        /*!
         \brief The icon for the action of reverting to a previous version of a document.
        */
        static const std::string DOCUMENT_REVERT;

        /*!
         \brief The icon for the save action. Should be an arrow pointing down and toward a hard disk.
        */
        static const std::string DOCUMENT_SAVE;

        /*!
         \brief The icon for the save as action.
        */
        static const std::string DOCUMENT_SAVE_AS;

        /*!
         \brief The icon for the send action. Should be an arrow pointing up and away from a hard disk.
        */
        static const std::string DOCUMENT_SEND;

        /*!
         \brief The icon for the clear action.
        */
        static const std::string EDIT_CLEAR;

        /*!
         \brief The icon for the copy action.
        */
        static const std::string EDIT_COPY;

        /*!
         \brief The icon for the cut action.
        */
        static const std::string EDIT_CUT;

        /*!
         \brief The icon for the delete action.
        */
        static const std::string EDIT_DELETE;

        /*!
         \brief The icon for the find action.
        */
        static const std::string EDIT_FIND;

        /*!
         \brief The icon for the find and replace action.
        */
        static const std::string EDIT_FIND_REPLACE;

        /*!
         \brief The icon for the paste action.
        */
        static const std::string EDIT_PASTE;

        /*!
         \brief The icon for the redo action.
        */
        static const std::string EDIT_REDO;

        /*!
         \brief The icon for the select all action.
        */
        static const std::string EDIT_SELECT_ALL;

        /*!
         \brief The icon for the undo action.
        */
        static const std::string EDIT_UNDO;

        /*!
         \brief The icon for creating a new folder.
        */
        static const std::string FOLDER_NEW;

        /*!
         \brief The icon for the decrease indent formatting action.
        */
        static const std::string FORMAT_INDENT_LESS;

        /*!
         \brief The icon for the increase indent formatting action.
        */
        static const std::string FORMAT_INDENT_MORE;

        /*!
         \brief The icon for the center justification formatting action.
        */
        static const std::string FORMAT_JUSTIFY_CENTER;

        /*!
         \brief The icon for the fill justification formatting action.
        */
        static const std::string FORMAT_JUSTIFY_FILL;

        /*!
         \brief The icon for the left justification formatting action.
        */
        static const std::string FORMAT_JUSTIFY_LEFT;

        /*!
         \brief The icon for the right justification action.
        */
        static const std::string FORMAT_JUSTIFY_RIGHT;

        /*!
         \brief The icon for the left-to-right text formatting action.
        */
        static const std::string FORMAT_TEXT_DIRECTION_LTR;

        /*!
         \brief The icon for the right-to-left formatting action.
        */
        static const std::string FORMAT_TEXT_DIRECTION_RTL;

        /*!
         \brief The icon for the bold text formatting action.
        */
        static const std::string FORMAT_TEXT_BOLD;

        /*!
         \brief The icon for the italic text formatting action.
        */
        static const std::string FORMAT_TEXT_ITALIC;

        /*!
         \brief The icon for the underlined text formatting action.
        */
        static const std::string FORMAT_TEXT_UNDERLINE;

        /*!
         \brief The icon for the strikethrough text formatting action.
        */
        static const std::string FORMAT_TEXT_STRIKETHROUGH;

        /*!
         \brief The icon for the go to bottom of a list action.
        */
        static const std::string GO_BOTTOM;

        /*!
         \brief The icon for the go down in a list action.
        */
        static const std::string GO_DOWN;

        /*!
         \brief The icon for the go to the first item in a list action.
        */
        static const std::string GO_FIRST;

        /*!
         \brief The icon for the go to home location action.
        */
        static const std::string GO_HOME;

        /*!
         \brief The icon for the jump to action.
        */
        static const std::string GO_JUMP;

        /*!
         \brief The icon for the go to the last item in a list action.
        */
        static const std::string GO_LAST;

        /*!
         \brief The icon for the go to the next item in a list action.
        */
        static const std::string GO_NEXT;

        /*!
         \brief The icon for the go to the previous item in a list action.
        */
        static const std::string GO_PREVIOUS;

        /*!
         \brief The icon for the go to the top of a list action.
        */
        static const std::string GO_TOP;

        /*!
         \brief The icon for the go up in a list action.
        */
        static const std::string GO_UP;

        /*!
         \brief The icon for the About item in the Help menu.
        */
        static const std::string HELP_ABOUT;

        /*!
         \brief The icon for Contents item in the Help menu.
        */
        static const std::string HELP_CONTENTS;

        /*!
         \brief The icon for the FAQ item in the Help menu.
        */
        static const std::string HELP_FAQ;

        /*!
         \brief The icon for the insert image action of an application.
        */
        static const std::string INSERT_IMAGE;

        /*!
         \brief The icon for the insert link action of an application.
        */
        static const std::string INSERT_LINK;

        /*!
         \brief The icon for the insert object action of an application.
        */
        static const std::string INSERT_OBJECT;

        /*!
         \brief The icon for the insert text action of an application.
        */
        static const std::string INSERT_TEXT;

        /*!
         \brief The icon for the add to list action.
        */
        static const std::string LIST_ADD;

        /*!
         \brief The icon for the remove from list action.
        */
        static const std::string LIST_REMOVE;

        /*!
         \brief The icon for the forward action of an electronic mail application.
        */
        static const std::string MAIL_FORWARD;

        /*!
         \brief The icon for the mark as important action of an electronic mail application.
        */
        static const std::string MAIL_MARK_IMPORTANT;

        /*!
         \brief The icon for the mark as junk action of an electronic mail application.
        */
        static const std::string MAIL_MARK_JUNK;

        /*!
         \brief The icon for the mark as not junk action of an electronic mail application.
        */
        static const std::string MAIL_MARK_NOTJUNK;

        /*!
         \brief The icon for the mark as read action of an electronic mail application.
        */
        static const std::string MAIL_MARK_READ;

        /*!
         \brief The icon for the mark as unread action of an electronic mail application.
        */
        static const std::string MAIL_MARK_UNREAD;

        /*!
         \brief The icon for the compose new mail action of an electronic mail application.
        */
        static const std::string MAIL_MESSAGE_NEW;

        /*!
         \brief The icon for the reply to all action of an electronic mail application.
        */
        static const std::string MAIL_REPLY_ALL;

        /*!
         \brief The icon for the reply to sender action of an electronic mail application.
        */
        static const std::string MAIL_REPLY_SENDER;

        /*!
         \brief The icon for the send action of an electronic mail application.
        */
        static const std::string MAIL_SEND;

        /*!
         \brief The icon for the send and receive action of an electronic mail application.
        */
        static const std::string MAIL_SEND_RECEIVE;

        /*!
         \brief The icon for the eject action of a media player or file manager.
        */
        static const std::string MEDIA_EJECT;

        /*!
         \brief The icon for the pause action of a media player.
        */
        static const std::string MEDIA_PLAYBACK_PAUSE;

        /*!
         \brief The icon for the start playback action of a media player.
        */
        static const std::string MEDIA_PLAYBACK_START;

        /*!
         \brief The icon for the stop action of a media player.
        */
        static const std::string MEDIA_PLAYBACK_STOP;

        /*!
         \brief The icon for the record action of a media application.
        */
        static const std::string MEDIA_RECORD;

        /*!
         \brief The icon for the seek backward action of a media player.
        */
        static const std::string MEDIA_SEEK_BACKWARD;

        /*!
         \brief The icon for the seek forward action of a media player.
        */
        static const std::string MEDIA_SEEK_FORWARD;

        /*!
         \brief The icon for the skip backward action of a media player.
        */
        static const std::string MEDIA_SKIP_BACKWARD;

        /*!
         \brief The icon for the skip forward action of a media player.
        */
        static const std::string MEDIA_SKIP_FORWARD;

        /*!
         \brief The icon for the action to flip an object horizontally.
        */
        static const std::string OBJECT_FLIP_HORIZONTAL;

        /*!
         \brief The icon for the action to flip an object vertically.
        */
        static const std::string OBJECT_FLIP_VERTICAL;

        /*!
         \brief The icon for the rotate left action performed on an object.
        */
        static const std::string OBJECT_ROTATE_LEFT;

        /*!
         \brief The icon for the rotate rigt action performed on an object.
        */
        static const std::string OBJECT_ROTATE_RIGHT;

        /*!
         \brief The icon used for the "Stop" action in applications with actions that may take a while to process, such as web page loading in a browser.
        */
        static const std::string PROCESS_STOP;

        /*!
         \brief The icon used for the "Lock Screen" item in the desktop's panel application.
        */
        static const std::string SYSTEM_LOCK_SCREEN;

        /*!
         \brief The icon used for the "Log Out" item in the desktop's panel application.
        */
        static const std::string SYSTEM_LOG_OUT;

        /*!
         \brief The icon used for the "Run Application..." item in the desktop's panel application.
        */
        static const std::string SYSTEM_RUN;

        /*!
         \brief The icon used for the "Search" item in the desktop's panel application.
        */
        static const std::string SYSTEM_SEARCH;

        /*!
         \brief The icon used for the "Reboot" item in the desktop's panel application.
        */
        static const std::string SYSTEM_REBOOT;

        /*!
         \brief The icon used for the "Shutdown" item in the desktop's panel application.
        */
        static const std::string SYSTEM_SHUTDOWN;

        /*!
         \brief The icon used for the "Check Spelling" item in the application's "Tools" menu.
        */
        static const std::string TOOLS_CHECK_SPELLING;

        /*!
         \brief The icon used for the "Fullscreen" item in the application's "View" menu.
        */
        static const std::string VIEW_FULLSCREEN;

        /*!
         \brief The icon used for the "Refresh" item in the application's "View" menu.
        */
        static const std::string VIEW_REFRESH;

        /*!
         \brief The icon used by an application for leaving the fullscreen view, and returning to a normal windowed view.
        */
        static const std::string VIEW_RESTORE;

        /*!
         \brief The icon used for the "Sort Ascending" item in the application's "View" menu, or in a button for changing the sort method for a list.
        */
        static const std::string VIEW_SORT_ASCENDING;

        /*!
         \brief The icon used for the "Sort Descending" item in the application's "View" menu, or in a button for changing the sort method for a list.
        */
        static const std::string VIEW_SORT_DESCENDING;

        /*!
         \brief The icon used for the "Close Window" item in the application's "Windows" menu.
        */
        static const std::string WINDOW_CLOSE;

        /*!
         \brief The icon used for the "New Window" item in the application's "Windows" menu.
        */
        static const std::string WINDOW_NEW;

        /*!
         \brief The icon used for the "Best Fit" item in the application's "View" menu.
        */
        static const std::string ZOOM_FIT_BEST;

        /*!
         \brief The icon used for the "Zoom in" item in the application's "View" menu.
        */
        static const std::string ZOOM_IN;

        /*!
         \brief The icon used for the "Original Size" item in the application's "View" menu.
        */
        static const std::string ZOOM_ORIGINAL;

        /*!
         \brief The icon used for the "Zoom Out" item in the application's "View" menu.
        */
        static const std::string ZOOM_OUT;
    };

    /*!
      \brief Animated images used to represent loading web sites, or other background processing which may be less suited to more verbose progress reporting in the user interface.
        Animations should be a PNG with frames which are the size of the directory the animation is in, tiled in a WxH grid. Implementations should determine the number of
        frames by dividing the image into its frames, and iterating from left to right, wrapping to the first frame, after rendering the last.
     */
    class IconNamingAnimations {
    private:
        IconNamingAnimations () {};

    public:
        /*!
         \brief This is the standard spinner animation for web browsers and file managers to show that the location is loading.
        */
        static const std::string PROCESS_WORKING;

    };

    /*!
      \brief Icons that describe what an application is, for use in the Programs menu, window decorations, and the task list.
        These may or may not be generic depending on the application and its purpose. Applications which are to be considered
        part of the base desktop, such as the calculator or terminal, should use the generic icons specified in this specification,
        while more advanced applications such as web browsers and office applications should use branded icons which still give
        the user an idea of what function the application provides.
     */
    class IconNamingApplications {
    private:
        IconNamingApplications () {};

    public:
        /*!
         \brief The icon used for the desktop's calculator accessory program.
        */
        static const std::string ACCESSORIES_CALCULATOR;

        /*!
         \brief The icon used for the desktop's international and extended text character accessory program.
        */
        static const std::string ACCESSORIES_CHARACTER_MAP;

        /*!
         \brief The icon used for the desktop's dictionary accessory program.
        */
        static const std::string ACCESSORIES_DICTIONARY;

        /*!
         \brief The icon used for the desktop's text editing accessory program.
        */
        static const std::string ACCESSORIES_TEXT_EDITOR;

        /*!
         \brief The icon used for the desktop's help browsing application.
        */
        static const std::string HELP_BROWSER;

        /*!
         \brief The icon used for the desktop's hardware volume control application.
        */
        static const std::string MULTIMEDIA_VOLUME_CONTROL;

        /*!
         \brief The icon used for the desktop's accessibility preferences.
        */
        static const std::string PREFERENCES_DESKTOP_ACCESSIBILITY;

        /*!
         \brief The icon used for the desktop's font preferences.
        */
        static const std::string PREFERENCES_DESKTOP_FONT;

        /*!
         \brief The icon used for the desktop's keyboard preferences.
        */
        static const std::string PREFERENCES_DESKTOP_KEYBOARD;

        /*!
         \brief The icon used for the desktop's locale preferences.
        */
        static const std::string PREFERENCES_DESKTOP_LOCALE;

        /*!
         \brief The icon used for the desktop's multimedia preferences.
        */
        static const std::string PREFERENCES_DESKTOP_MULTIMEDIA;

        /*!
         \brief The icon used for the desktop's screen saving preferences.
        */
        static const std::string PREFERENCES_DESKTOP_SCREENSAVER;

        /*!
         \brief The icon used for the desktop's theme preferences.
        */
        static const std::string PREFERENCES_DESKTOP_THEME;

        /*!
         \brief The icon used for the desktop's wallpaper preferences.
        */
        static const std::string PREFERENCES_DESKTOP_WALLPAPER;

        /*!
         \brief The icon used for the desktop's file management application.
        */
        static const std::string SYSTEM_FILE_MANAGER;

        /*!
         \brief The icon used for the desktop's software installer application.
        */
        static const std::string SYSTEM_SOFTWARE_INSTALL;

        /*!
         \brief The icon used for the desktop's software updating application.
        */
        static const std::string SYSTEM_SOFTWARE_UPDATE;

        /*!
         \brief The icon used for the desktop's system resource monitor application.
        */
        static const std::string UTILITIES_SYSTEM_MONITOR;

        /*!
         \brief The icon used for the desktop's terminal emulation application.
        */
        static const std::string UTILITIES_TERMINAL;
    };

    /*!
     \brief Icons that are used for categories in the Programs menu, or the Control Center, for separating applications,
            preferences, and settings for display to the user.
    */
    class IconNamingCategories {
    private:
        IconNamingCategories() {};

    public:
        /*!
         \brief The icon for the "Accessories" sub-menu of the Programs menu.
        */
        static const std::string APPLICATIONS_ACCESSORIES;

        /*!
         \brief The icon for the "Programming" sub-menu of the Programs menu.
        */
        static const std::string APPLICATIONS_DEVELOPMENT;

        /*!
         \brief The icon for the "Engineering" sub-menu of the Programs menu.
        */
        static const std::string APPLICATIONS_ENGINEERING;

        /*!
         \brief The icon for the "Games" sub-menu of the Programs menu.
        */
        static const std::string APPLICATIONS_GAMES;

        /*!
         \brief The icon for the "Graphics" sub-menu of the Programs menu.
        */
        static const std::string APPLICATIONS_GRAPHICS;

        /*!
         \brief The icon for the "Internet" sub-menu of the Programs menu.
        */
        static const std::string APPLICATIONS_INTERNET;

        /*!
         \brief The icon for the "Multimedia" sub-menu of the Programs menu.
        */
        static const std::string APPLICATIONS_MULTIMEDIA;

        /*!
         \brief The icon for the "Office" sub-menu of the Programs menu.
        */
        static const std::string APPLICATIONS_OFFICE;

        /*!
         \brief The icon for the "Other" sub-menu of the Programs menu.
        */
        static const std::string APPLICATIONS_OTHER;

        /*!
         \brief The icon for the "Science" sub-menu of the Programs menu.
        */
        static const std::string APPLICATIONS_SCIENCE;

        /*!
         \brief The icon for the "System Tools" sub-menu of the Programs menu.
        */
        static const std::string APPLICATIONS_SYSTEM;

        /*!
         \brief The icon for the "Utilities" sub-menu of the Programs menu.
        */
        static const std::string APPLICATIONS_UTILITIES;

        /*!
         \brief The icon for the "Desktop Preferences" category.
        */
        static const std::string PREFERENCES_DESKTOP;

        /*!
         \brief The icon for the "Peripherals" sub-category of the "Desktop Preferences" category.
        */
        static const std::string PREFERENCES_DESKTOP_PERIPHERALS;

        /*!
         \brief The icon for the "Personal" sub-category of the "Desktop Preferences" category.
        */
        static const std::string PREFERENCES_DESKTOP_PERSONAL;

        /*!
         \brief The icon for the "Other" preferences category.
        */
        static const std::string PREFERENCES_OTHER;

        /*!
         \brief The icon for the "System Preferences" category.
        */
        static const std::string PREFERENCES_SYSTEM;

        /*!
         \brief The icon for the "Network" sub-category of the "System Preferences" category.
        */
        static const std::string PREFERENCES_SYSTEM_NETWORK;

        /*!
         \brief The icon for the "Help" system category.
        */
        static const std::string SYSTEM_HELP;
    };

    /*!
      \brief Icons for hardware that is contained within or connected to the computing device.
      \par Description
        Naming for extended devices in this group, is of the form \<primary function\>-\<manufacturer\>-\<model\>. This allows ease of fallback
        to the primary function device name, or ones more targeted for a specific series of models from a manufacturer. For example,
        a theme author may want to provide icons for different phones. The specific model icons could be named "phone-samsung-t809",
        "phone-motorola-rokr", and "phone-motorola-pebl".
        However, the theme must provide a phone icon in the theme's style, so that devices not matching these models, will still
        have an appropriate icon. An exception to this rule is that the "media" icons do not need to include manufacturer names,
        as they are generic items, and may be available from many manufacturers. As a result, for media, the specific icons are to
        differentiate between different specific types of media. For exmaple, an artist may wish to provide icons for BluRay, DVD, HD-DVD, CD-ROM,
        and variations thereof. The specific media type icons should be named in the form, \<primary function\>-\<specific format\>.
        Some examples are "media-optical", "media-optical-bd" and "media-optical-dvd".
     */
    class IconNamingDevices {
    private:
        IconNamingDevices() {};

    public:
        /*!
         \brief The icon used for the audio rendering device.
        */
        static const std::string AUDIO_CARD;

        /*!
         \brief The icon used for the microphone audio input device.
        */
        static const std::string AUDIO_INPUT_MICROPHONE;

        /*!
         \brief The icon used for the system battery device.
        */
        static const std::string BATTERY;

        /*!
         \brief The icon used for a digital still camera devices.
        */
        static const std::string CAMERA_PHOTO;

        /*!
         \brief The fallback icon for video cameras.
        */
        static const std::string CAMERA_VIDEO;

        /*!
         \brief The fallback icon for web cameras.
        */
        static const std::string CAMERA_WEB;

        /*!
         \brief The icon used for the computing device as a whole.
        */
        static const std::string COMPUTER;

        /*!
         \brief The icon used for hard disk drives.
        */
        static const std::string DRIVE_HARDDISK;

        /*!
         \brief The icon used for optical media drives such as CD and DVD.
        */
        static const std::string DRIVE_OPTICAL;

        /*!
         \brief The icon used for removable media drives.
        */
        static const std::string DRIVE_REMOVABLE_MEDIA;

        /*!
         \brief The icon used for the gaming input device.
        */
        static const std::string INPUT_GAMING;

        /*!
         \brief The icon used for the keyboard input device.
        */
        static const std::string INPUT_KEYBOARD;

        /*!
         \brief The icon used for the mousing input device.
        */
        static const std::string INPUT_MOUSE;

        /*!
         \brief The icon used for graphics tablet input devices.
        */
        static const std::string INPUT_TABLET;

        /*!
         \brief The fallback icon used for flash media, such as memory stick and SD.
        */
        static const std::string MEDIA_FLASH;

        /*!
         \brief The icon used for physical floppy disk media.
        */
        static const std::string MEDIA_FLOPPY;

        /*!
         \brief The icon used for physical optical media such as CD and DVD.
        */
        static const std::string MEDIA_OPTICAL;

        /*!
         \brief The icon used for generic physical tape media.
        */
        static const std::string MEDIA_TAPE;

        /*!
         \brief The icon used for modem devices.
        */
        static const std::string MODEM;

        /*!
         \brief The icon used for generic multimedia playing devices.
        */
        static const std::string MULTIMEDIA_PLAYER;

        /*!
         \brief The icon used for wired network connections.
        */
        static const std::string NETWORK_WIRED;

        /*!
         \brief The icon used for wireless network connections.
        */
        static const std::string NETWORK_WIRELESS;

        /*!
         \brief This is the fallback icon for Personal Digial Assistant devices. Primary use of this icon is for PDA devices connected to the PC.
            Connection medium is not an important aspect of the icon. The metaphor for this fallback icon should be a generic PDA device icon.
        */
        static const std::string PDA;

        /*!
         \brief This is the default fallback for phone devices. Primary use of this icon group is for phone devices which support connectivity to the PC.
            These may be VoIP, cellular, or possibly landline phones. The metaphor for this fallback should be a generic mobile phone device.
        */
        static const std::string PHONE;

        /*!
         \brief The icon used for a printer device.
        */
        static const std::string PRINTER;

        /*!
         \brief The icon used for a scanner device.
        */
        static const std::string SCANNER;

        /*!
         \brief The icon used for the monitor that video gets displayed to.
        */
        static const std::string VIDEO_DISPLAY;

    };

    /*!
      \brief Icons for tags and properties of files, that are displayed in the file manager. This context contains emblems for such things as “read-only” or “photos”.
     */
    class IconNamingEmblems {
    private:
        IconNamingEmblems() {};

    public:
        /*!
         \brief The icon used as an emblem to specify the default selection of a printer for example.
        */
        static const std::string EMBLEM_DEFAULT;

        /*!
         \brief The icon used as an emblem for the directory where a user's documents are stored.
        */
        static const std::string EMBLEM_DOCUMENTS;

        /*!
         \brief The icon used as an emblem for the directory where a user's downloads from the internet are stored.
        */
        static const std::string EMBLEM_DOWNLOADS;

        /*!
         \brief The icon used as an emblem for files and directories that the user marks as favorites.
        */
        static const std::string EMBLEM_FAVORITE;

        /*!
         \brief The icon used as an emblem for files and directories that are marked as important by the user.
        */
        static const std::string EMBLEM_IMPORTANT;

        /*!
         \brief The icon used as an emblem to specify the directory where the user's electronic mail is stored.
        */
        static const std::string EMBLEM_MAIL;

        /*!
         \brief The icon used as an emblem to specify the directory where the user stores photographs.
        */
        static const std::string EMBLEM_PHOTOS;

        /*!
         \brief The icon used as an emblem for files and directories which can not be written to by the user.
        */
        static const std::string EMBLEM_READONLY;

        /*!
         \brief The icon used as an emblem for files and directories that are shared to other users.
        */
        static const std::string EMBLEM_SHARED;

        /*!
         \brief The icon used as an emblem for files and direcotires that are links to other files or directories on the filesystem.
        */
        static const std::string EMBLEM_SYMBOLIC_LINK;

        /*!
         \brief The icon used as an emblem for files or directories that are configured to be synchronized to another device.
        */
        static const std::string EMBLEM_SYNCHRONIZED;

        /*!
         \brief The icon used as an emblem for directories that contain system libraries, settings, and data.
        */
        static const std::string EMBLEM_SYSTEM;

        /*!
         \brief The icon used as an emblem for files and directories that are inaccessible.
        */
        static const std::string EMBLEM_UNREADABLE;
    };

    /*!
      \brief Icons for emotions that are expressed through text chat applications such as :-) or :-P in IRC or instant messengers.
     */
    class IconNamingEmotes {
    private:
        IconNamingEmotes() {};

    public:
        /*!
         \brief The icon used for the 0:-) emote.
        */
        static const std::string FACE_ANGEL;

        /*!
         \brief The icon used for the X-( emote.
        */
        static const std::string FACE_ANGRY;

        /*!
         \brief The icon used for the B-) emote.
        */
        static const std::string FACE_COOL;

        /*!
         \brief The icon used for the :'( emote.
        */
        static const std::string FACE_CRYING;

        /*!
         \brief The icon used for the >:-) emote.
        */
        static const std::string FACE_DEVILISH;

        /*!
         \brief The icon used for the :-[ emote.
        */
        static const std::string FACE_EMBARRASSED;

        /*!
         \brief The icon used for the :-* emote.
        */
        static const std::string FACE_KISS;

        /*!
         \brief The icon used for the :-)) emote.
        */
        static const std::string FACE_LAUGH;

        /*!
         \brief The icon used for the :-(|) emote.
        */
        static const std::string FACE_MONKEY;

        /*!
         \brief The icon used for the :-| emote.
        */
        static const std::string FACE_PLAIN;

        /*!
         \brief The icon used for the :-P emote.
        */
        static const std::string FACE_RASPBERRY;

        /*!
         \brief The icon used for the :-( emote.
        */
        static const std::string FACE_SAD;

        /*!
         \brief The icon used for the :-& emote.
        */
        static const std::string FACE_SICK;

        /*!
         \brief The icon used for the :-) emote.
        */
        static const std::string FACE_SMILE;

        /*!
         \brief The icon used for the :-D emote.
        */
        static const std::string FACE_SMILE_BIG;

        /*!
         \brief The icon used for the :-! emote.
        */
        static const std::string FACE_SMIRK;

        /*!
         \brief The icon used for the :-0 emote.
        */
        static const std::string FACE_SURPRISE;

        /*!
         \brief The icon used for the |-) emote.
        */
        static const std::string FACE_TIRED;

        /*!
         \brief The icon used for the :-/ emote.
        */
        static const std::string FACE_UNCERTAIN;

        /*!
         \brief The icon used for the ;-) emote.
        */
        static const std::string FACE_WINK;

        /*!
         \brief The icon used for the :-S emote.
        */
        static const std::string FACE_WORRIED;

    };

    /*!
      \brief Icons for international denominations such as flags.
     */
    class IconNamingInternational {
    private:
        IconNamingInternational() {};

    public:
        /*!
         \brief AFGHANISTAN
        */
        static const std::string FLAG_AF;

        /*!
         \brief ÅLAND ISLANDS
        */
        static const std::string FLAG_AX;

        /*!
         \brief ALBANIA
        */
        static const std::string FLAG_AL;

        /*!
         \brief ALGERIA
        */
        static const std::string FLAG_DZ;

        /*!
         \brief AMERICAN SAMOA
        */
        static const std::string FLAG_AS;

        /*!
         \brief ANDORRA
        */
        static const std::string FLAG_AD;

        /*!
         \brief ANGOLA
        */
        static const std::string FLAG_AO;

        /*!
         \brief ANGUILLA
        */
        static const std::string FLAG_AI;

        /*!
         \brief ANTARCTICA
        */
        static const std::string FLAG_AQ;

        /*!
         \brief ANTIGUA AND BARBUDA
        */
        static const std::string FLAG_AG;

        /*!
         \brief ARGENTINA
        */
        static const std::string FLAG_AR;

        /*!
         \brief ARMENIA
        */
        static const std::string FLAG_AM;

        /*!
         \brief ARUBA
        */
        static const std::string FLAG_AW;

        /*!
         \brief AUSTRALIA
        */
        static const std::string FLAG_AU;

        /*!
         \brief AUSTRIA
        */
        static const std::string FLAG_AT;

        /*!
         \brief AZERBAIJAN
        */
        static const std::string FLAG_AZ;

        /*!
         \brief BAHAMAS
        */
        static const std::string FLAG_BS;

        /*!
         \brief BAHRAIN
        */
        static const std::string FLAG_BH;

        /*!
         \brief BANGLADESH
        */
        static const std::string FLAG_BD;

        /*!
         \brief BARBADOS
        */
        static const std::string FLAG_BB;

        /*!
         \brief BELARUS
        */
        static const std::string FLAG_BY;

        /*!
         \brief BELGIUM
        */
        static const std::string FLAG_BE;

        /*!
         \brief BELIZE
        */
        static const std::string FLAG_BZ;

        /*!
         \brief BENIN
        */
        static const std::string FLAG_BJ;

        /*!
         \brief BERMUDA
        */
        static const std::string FLAG_BM;

        /*!
         \brief BHUTAN
        */
        static const std::string FLAG_BT;

        /*!
         \brief BOLIVIA, PLURINATIONAL STATE OF
        */
        static const std::string FLAG_BO;

        /*!
         \brief BOSNIA AND HERZEGOVINA
        */
        static const std::string FLAG_BA;

        /*!
         \brief BOTSWANA
        */
        static const std::string FLAG_BW;

        /*!
         \brief BOUVET ISLAND
        */
        static const std::string FLAG_BV;

        /*!
         \brief BRAZIL
        */
        static const std::string FLAG_BR;

        /*!
         \brief BRITISH INDIAN OCEAN TERRITORY
        */
        static const std::string FLAG_IO;

        /*!
         \brief BRUNEI DARUSSALAM
        */
        static const std::string FLAG_BN;

        /*!
         \brief BULGARIA
        */
        static const std::string FLAG_BG;

        /*!
         \brief BURKINA FASO
        */
        static const std::string FLAG_BF;

        /*!
         \brief BURUNDI
        */
        static const std::string FLAG_BI;

        /*!
         \brief CAMBODIA
        */
        static const std::string FLAG_KH;

        /*!
         \brief CAMEROON
        */
        static const std::string FLAG_CM;

        /*!
         \brief CANADA
        */
        static const std::string FLAG_CA;

        /*!
         \brief CAPE VERDE
        */
        static const std::string FLAG_CV;

        /*!
         \brief CAYMAN ISLANDS
        */
        static const std::string FLAG_KY;

        /*!
         \brief CENTRAL AFRICAN REPUBLIC
        */
        static const std::string FLAG_CF;

        /*!
         \brief CHAD
        */
        static const std::string FLAG_TD;

        /*!
         \brief CHILE
        */
        static const std::string FLAG_CL;

        /*!
         \brief CHINA
        */
        static const std::string FLAG_CN;

        /*!
         \brief CHRISTMAS ISLAND
        */
        static const std::string FLAG_CX;

        /*!
         \brief COCOS (KEELING) ISLANDS
        */
        static const std::string FLAG_CC;

        /*!
         \brief COLOMBIA
        */
        static const std::string FLAG_CO;

        /*!
         \brief COMOROS
        */
        static const std::string FLAG_KM;

        /*!
         \brief CONGO
        */
        static const std::string FLAG_CG;

        /*!
         \brief CONGO, THE DEMOCRATIC REPUBLIC OF THE
        */
        static const std::string FLAG_CD;

        /*!
         \brief COOK ISLANDS
        */
        static const std::string FLAG_CK;

        /*!
         \brief COSTA RICA
        */
        static const std::string FLAG_CR;

        /*!
         \brief CÔTE D'IVOIRE
        */
        static const std::string FLAG_CI;

        /*!
         \brief CROATIA
        */
        static const std::string FLAG_HR;

        /*!
         \brief CUBA
        */
        static const std::string FLAG_CU;

        /*!
         \brief CYPRUS
        */
        static const std::string FLAG_CY;

        /*!
         \brief CZECH REPUBLIC
        */
        static const std::string FLAG_CZ;

        /*!
         \brief DENMARK
        */
        static const std::string FLAG_DK;

        /*!
         \brief DJIBOUTI
        */
        static const std::string FLAG_DJ;

        /*!
         \brief DOMINICA
        */
        static const std::string FLAG_DM;

        /*!
         \brief DOMINICAN REPUBLIC
        */
        static const std::string FLAG_DO;

        /*!
         \brief ECUADOR
        */
        static const std::string FLAG_EC;

        /*!
         \brief EGYPT
        */
        static const std::string FLAG_EG;

        /*!
         \brief EL SALVADOR
        */
        static const std::string FLAG_SV;

        /*!
         \brief EQUATORIAL GUINEA
        */
        static const std::string FLAG_GQ;

        /*!
         \brief ERITREA
        */
        static const std::string FLAG_ER;

        /*!
         \brief ESTONIA
        */
        static const std::string FLAG_EE;

        /*!
         \brief ETHIOPIA
        */
        static const std::string FLAG_ET;

        /*!
         \brief FALKLAND ISLANDS (MALVINAS)
        */
        static const std::string FLAG_FK;

        /*!
         \brief FAROE ISLANDS
        */
        static const std::string FLAG_FO;

        /*!
         \brief FIJI
        */
        static const std::string FLAG_FJ;

        /*!
         \brief FINLAND
        */
        static const std::string FLAG_FI;

        /*!
         \brief FRANCE
        */
        static const std::string FLAG_FR;

        /*!
         \brief FRENCH GUIANA
        */
        static const std::string FLAG_GF;

        /*!
         \brief FRENCH POLYNESIA
        */
        static const std::string FLAG_PF;

        /*!
         \brief FRENCH SOUTHERN TERRITORIES
        */
        static const std::string FLAG_TF;

        /*!
         \brief GABON
        */
        static const std::string FLAG_GA;

        /*!
         \brief GAMBIA
        */
        static const std::string FLAG_GM;

        /*!
         \brief GEORGIA
        */
        static const std::string FLAG_GE;

        /*!
         \brief GERMANY
        */
        static const std::string FLAG_DE;

        /*!
         \brief GHANA
        */
        static const std::string FLAG_GH;

        /*!
         \brief GIBRALTAR
        */
        static const std::string FLAG_GI;

        /*!
         \brief GREECE
        */
        static const std::string FLAG_GR;

        /*!
         \brief GREENLAND
        */
        static const std::string FLAG_GL;

        /*!
         \brief GRENADA
        */
        static const std::string FLAG_GD;

        /*!
         \brief GUADELOUPE
        */
        static const std::string FLAG_GP;

        /*!
         \brief GUAM
        */
        static const std::string FLAG_GU;

        /*!
         \brief GUATEMALA
        */
        static const std::string FLAG_GT;

        /*!
         \brief GUERNSEY
        */
        static const std::string FLAG_GG;

        /*!
         \brief GUINEA
        */
        static const std::string FLAG_GN;

        /*!
         \brief GUINEA-BISSAU
        */
        static const std::string FLAG_GW;

        /*!
         \brief GUYANA
        */
        static const std::string FLAG_GY;

        /*!
         \brief HAITI
        */
        static const std::string FLAG_HT;

        /*!
         \brief HEARD ISLAND AND MCDONALD ISLANDS
        */
        static const std::string FLAG_HM;

        /*!
         \brief HOLY SEE (VATICAN CITY STATE)
        */
        static const std::string FLAG_VA;

        /*!
         \brief HONDURAS
        */
        static const std::string FLAG_HN;

        /*!
         \brief HONG KONG
        */
        static const std::string FLAG_HK;

        /*!
         \brief HUNGARY
        */
        static const std::string FLAG_HU;

        /*!
         \brief ICELAND
        */
        static const std::string FLAG_IS;

        /*!
         \brief INDIA
        */
        static const std::string FLAG_IN;

        /*!
         \brief INDONESIA
        */
        static const std::string FLAG_ID;

        /*!
         \brief IRAN, ISLAMIC REPUBLIC OF
        */
        static const std::string FLAG_IR;

        /*!
         \brief IRAQ
        */
        static const std::string FLAG_IQ;

        /*!
         \brief IRELAND
        */
        static const std::string FLAG_IE;

        /*!
         \brief ISLE OF MAN
        */
        static const std::string FLAG_IM;

        /*!
         \brief ISRAEL
        */
        static const std::string FLAG_IL;

        /*!
         \brief ITALY
        */
        static const std::string FLAG_IT;

        /*!
         \brief JAMAICA
        */
        static const std::string FLAG_JM;

        /*!
         \brief JAPAN
        */
        static const std::string FLAG_JP;

        /*!
         \brief JERSEY
        */
        static const std::string FLAG_JE;

        /*!
         \brief JORDAN
        */
        static const std::string FLAG_JO;

        /*!
         \brief KAZAKHSTAN
        */
        static const std::string FLAG_KZ;

        /*!
         \brief KENYA
        */
        static const std::string FLAG_KE;

        /*!
         \brief KIRIBATI
        */
        static const std::string FLAG_KI;

        /*!
         \brief KOREA, DEMOCRATIC PEOPLE'S REPUBLIC OF
        */
        static const std::string FLAG_KP;

        /*!
         \brief KOREA, REPUBLIC OF
        */
        static const std::string FLAG_KR;

        /*!
         \brief KUWAIT
        */
        static const std::string FLAG_KW;

        /*!
         \brief KYRGYZSTAN
        */
        static const std::string FLAG_KG;

        /*!
         \brief LAO PEOPLE'S DEMOCRATIC REPUBLIC
        */
        static const std::string FLAG_LA;

        /*!
         \brief LATVIA
        */
        static const std::string FLAG_LV;

        /*!
         \brief LEBANON
        */
        static const std::string FLAG_LB;

        /*!
         \brief LESOTHO
        */
        static const std::string FLAG_LS;

        /*!
         \brief LIBERIA
        */
        static const std::string FLAG_LR;

        /*!
         \brief LIBYAN ARAB JAMAHIRIYA
        */
        static const std::string FLAG_LY;

        /*!
         \brief LIECHTENSTEIN
        */
        static const std::string FLAG_LI;

        /*!
         \brief LITHUANIA
        */
        static const std::string FLAG_LT;

        /*!
         \brief LUXEMBOURG
        */
        static const std::string FLAG_LU;

        /*!
         \brief MACAO
        */
        static const std::string FLAG_MO;

        /*!
         \brief MACEDONIA, THE FORMER YUGOSLAV REPUBLIC OF
        */
        static const std::string FLAG_MK;

        /*!
         \brief MADAGASCAR
        */
        static const std::string FLAG_MG;

        /*!
         \brief MALAWI
        */
        static const std::string FLAG_MW;

        /*!
         \brief MALAYSIA
        */
        static const std::string FLAG_MY;

        /*!
         \brief MALDIVES
        */
        static const std::string FLAG_MV;

        /*!
         \brief MALI
        */
        static const std::string FLAG_ML;

        /*!
         \brief MALTA
        */
        static const std::string FLAG_MT;

        /*!
         \brief MARSHALL ISLANDS
        */
        static const std::string FLAG_MH;

        /*!
         \brief MARTINIQUE
        */
        static const std::string FLAG_MQ;

        /*!
         \brief MAURITANIA
        */
        static const std::string FLAG_MR;

        /*!
         \brief MAURITIUS
        */
        static const std::string FLAG_MU;

        /*!
         \brief MAYOTTE
        */
        static const std::string FLAG_YT;

        /*!
         \brief MEXICO
        */
        static const std::string FLAG_MX;

        /*!
         \brief MICRONESIA, FEDERATED STATES OF
        */
        static const std::string FLAG_FM;

        /*!
         \brief MOLDOVA, REPUBLIC OF
        */
        static const std::string FLAG_MD;

        /*!
         \brief MONACO
        */
        static const std::string FLAG_MC;

        /*!
         \brief MONGOLIA
        */
        static const std::string FLAG_MN;

        /*!
         \brief MONTENEGRO
        */
        static const std::string FLAG_ME;

        /*!
         \brief MONTSERRAT
        */
        static const std::string FLAG_MS;

        /*!
         \brief MOROCCO
        */
        static const std::string FLAG_MA;

        /*!
         \brief MOZAMBIQUE
        */
        static const std::string FLAG_MZ;

        /*!
         \brief MYANMAR
        */
        static const std::string FLAG_MM;

        /*!
         \brief NAMIBIA
        */
        static const std::string FLAG_NA;

        /*!
         \brief NAURU
        */
        static const std::string FLAG_NR;

        /*!
         \brief NEPAL
        */
        static const std::string FLAG_NP;

        /*!
         \brief NETHERLANDS
        */
        static const std::string FLAG_NL;

        /*!
         \brief NETHERLANDS ANTILLES
        */
        static const std::string FLAG_AN;

        /*!
         \brief NEW CALEDONIA
        */
        static const std::string FLAG_NC;

        /*!
         \brief NEW ZEALAND
        */
        static const std::string FLAG_NZ;

        /*!
         \brief NICARAGUA
        */
        static const std::string FLAG_NI;

        /*!
         \brief NIGER
        */
        static const std::string FLAG_NE;

        /*!
         \brief NIGERIA
        */
        static const std::string FLAG_NG;

        /*!
         \brief NIUE
        */
        static const std::string FLAG_NU;

        /*!
         \brief NORFOLK ISLAND
        */
        static const std::string FLAG_NF;

        /*!
         \brief NORTHERN MARIANA ISLANDS
        */
        static const std::string FLAG_MP;

        /*!
         \brief NORWAY
        */
        static const std::string FLAG_NO;

        /*!
         \brief OMAN
        */
        static const std::string FLAG_OM;

        /*!
         \brief PAKISTAN
        */
        static const std::string FLAG_PK;

        /*!
         \brief PALAU
        */
        static const std::string FLAG_PW;

        /*!
         \brief PALESTINIAN TERRITORY, OCCUPIED
        */
        static const std::string FLAG_PS;

        /*!
         \brief PANAMA
        */
        static const std::string FLAG_PA;

        /*!
         \brief PAPUA NEW GUINEA
        */
        static const std::string FLAG_PG;

        /*!
         \brief PARAGUAY
        */
        static const std::string FLAG_PY;

        /*!
         \brief PERU
        */
        static const std::string FLAG_PE;

        /*!
         \brief PHILIPPINES
        */
        static const std::string FLAG_PH;

        /*!
         \brief PITCAIRN
        */
        static const std::string FLAG_PN;

        /*!
         \brief POLAND
        */
        static const std::string FLAG_PL;

        /*!
         \brief PORTUGAL
        */
        static const std::string FLAG_PT;

        /*!
         \brief PUERTO RICO
        */
        static const std::string FLAG_PR;

        /*!
         \brief QATAR
        */
        static const std::string FLAG_QA;

        /*!
         \brief RÉUNION
        */
        static const std::string FLAG_RE;

        /*!
         \brief ROMANIA
        */
        static const std::string FLAG_RO;

        /*!
         \brief RUSSIAN FEDERATION
        */
        static const std::string FLAG_RU;

        /*!
         \brief RWANDA
        */
        static const std::string FLAG_RW;

        /*!
         \brief SAINT BARTHÉLEMY
        */
        static const std::string FLAG_BL;

        /*!
         \brief SAINT HELENA, ASCENSION AND TRISTAN DA CUNHA
        */
        static const std::string FLAG_SH;

        /*!
         \brief SAINT KITTS AND NEVIS
        */
        static const std::string FLAG_KN;

        /*!
         \brief SAINT LUCIA
        */
        static const std::string FLAG_LC;

        /*!
         \brief SAINT MARTIN
        */
        static const std::string FLAG_MF;

        /*!
         \brief SAINT PIERRE AND MIQUELON
        */
        static const std::string FLAG_PM;

        /*!
         \brief SAINT VINCENT AND THE GRENADINES
        */
        static const std::string FLAG_VC;

        /*!
         \brief SAMOA
        */
        static const std::string FLAG_WS;

        /*!
         \brief SAN MARINO
        */
        static const std::string FLAG_SM;

        /*!
         \brief SAO TOME AND PRINCIPE
        */
        static const std::string FLAG_ST;

        /*!
         \brief SAUDI ARABIA
        */
        static const std::string FLAG_SA;

        /*!
         \brief SENEGAL
        */
        static const std::string FLAG_SN;

        /*!
         \brief SERBIA
        */
        static const std::string FLAG_RS;

        /*!
         \brief SEYCHELLES
        */
        static const std::string FLAG_SC;

        /*!
         \brief SIERRA LEONE
        */
        static const std::string FLAG_SL;

        /*!
         \brief SINGAPORE
        */
        static const std::string FLAG_SG;

        /*!
         \brief SLOVAKIA
        */
        static const std::string FLAG_SK;

        /*!
         \brief SLOVENIA
        */
        static const std::string FLAG_SI;

        /*!
         \brief SOLOMON ISLANDS
        */
        static const std::string FLAG_SB;

        /*!
         \brief SOMALIA
        */
        static const std::string FLAG_SO;

        /*!
         \brief SOUTH AFRICA
        */
        static const std::string FLAG_ZA;

        /*!
         \brief SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS
        */
        static const std::string FLAG_GS;

        /*!
         \brief SPAIN
        */
        static const std::string FLAG_ES;

        /*!
         \brief SRI LANKA
        */
        static const std::string FLAG_LK;

        /*!
         \brief SUDAN
        */
        static const std::string FLAG_SD;

        /*!
         \brief SURINAME
        */
        static const std::string FLAG_SR;

        /*!
         \brief SVALBARD AND JAN MAYEN
        */
        static const std::string FLAG_SJ;

        /*!
         \brief SWAZILAND
        */
        static const std::string FLAG_SZ;

        /*!
         \brief SWEDEN
        */
        static const std::string FLAG_SE;

        /*!
         \brief SWITZERLAND
        */
        static const std::string FLAG_CH;

        /*!
         \brief SYRIAN ARAB REPUBLIC
        */
        static const std::string FLAG_SY;

        /*!
         \brief TAIWAN, PROVINCE OF CHINA
        */
        static const std::string FLAG_TW;

        /*!
         \brief TAJIKISTAN
        */
        static const std::string FLAG_TJ;

        /*!
         \brief TANZANIA, UNITED REPUBLIC OF
        */
        static const std::string FLAG_TZ;

        /*!
         \brief THAILAND
        */
        static const std::string FLAG_TH;

        /*!
         \brief TIMOR-LESTE
        */
        static const std::string FLAG_TL;

        /*!
         \brief TOGO
        */
        static const std::string FLAG_TG;

        /*!
         \brief TOKELAU
        */
        static const std::string FLAG_TK;

        /*!
         \brief TONGA
        */
        static const std::string FLAG_TO;

        /*!
         \brief TRINIDAD AND TOBAGO
        */
        static const std::string FLAG_TT;

        /*!
         \brief TUNISIA
        */
        static const std::string FLAG_TN;

        /*!
         \brief TURKEY
        */
        static const std::string FLAG_TR;

        /*!
         \brief TURKMENISTAN
        */
        static const std::string FLAG_TM;

        /*!
         \brief TURKS AND CAICOS ISLANDS
        */
        static const std::string FLAG_TC;

        /*!
         \brief TUVALU
        */
        static const std::string FLAG_TV;

        /*!
         \brief UGANDA
        */
        static const std::string FLAG_UG;

        /*!
         \brief UKRAINE
        */
        static const std::string FLAG_UA;

        /*!
         \brief UNITED ARAB EMIRATES
        */
        static const std::string FLAG_AE;

        /*!
         \brief UNITED KINGDOM
        */
        static const std::string FLAG_GB;

        /*!
         \brief UNITED STATES
        */
        static const std::string FLAG_US;

        /*!
         \brief UNITED STATES MINOR OUTLYING ISLANDS
        */
        static const std::string FLAG_UM;

        /*!
         \brief URUGUAY
        */
        static const std::string FLAG_UY;

        /*!
         \brief UZBEKISTAN
        */
        static const std::string FLAG_UZ;

        /*!
         \brief VANUATU
        */
        static const std::string FLAG_VU;

        /*!
         \brief VENEZUELA, BOLIVARIAN REPUBLIC OF
        */
        static const std::string FLAG_VE;

        /*!
         \brief VIET NAM
        */
        static const std::string FLAG_VN;

        /*!
         \brief VIRGIN ISLANDS, BRITISH
        */
        static const std::string FLAG_VG;

        /*!
         \brief VIRGIN ISLANDS, U.S.
        */
        static const std::string FLAG_VI;

        /*!
         \brief WALLIS AND FUTUNA
        */
        static const std::string FLAG_WF;

        /*!
         \brief WESTERN SAHARA
        */
        static const std::string FLAG_EH;

        /*!
         \brief YEMEN
        */
        static const std::string FLAG_YE;

        /*!
         \brief ZAMBIA
        */
        static const std::string FLAG_ZM;

        /*!
         \brief ZIMBABWE
        */
        static const std::string FLAG_ZW;

    };

    /*!
      \brief Icons for different types of data, such as audio or image files.
     */
    class IconNamingMimeTypes {
    private:
        IconNamingMimeTypes() {};

    public:
        /*!
         \brief The icon used for generic binary file types.
         \note Not in the freedesktop naming spec.
        */
        static const std::string APPLICATION_X_GENERIC;

        /*!
         \brief The icon used for executable file types.
        */
        static const std::string APPLICATION_X_EXECUTABLE;

        /*!
         \brief The icon used for generic audio file types.
        */
        static const std::string AUDIO_X_GENERIC;

        /*!
         \brief The icon used for generic font file types.
        */
        static const std::string FONT_X_GENERIC;

        /*!
         \brief The icon used for generic image file types.
        */
        static const std::string IMAGE_X_GENERIC;

        /*!
         \brief The icon used for generic package file types.
        */
        static const std::string PACKAGE_X_GENERIC;

        /*!
         \brief The icon used for HTML text file types.
        */
        static const std::string TEXT_HTML;

        /*!
         \brief The icon used for generic text file types.
        */
        static const std::string TEXT_X_GENERIC;

        /*!
         \brief The icon used for generic text templates.
        */
        static const std::string TEXT_X_GENERIC_TEMPLATE;

        /*!
         \brief The icon used for script file types, such as shell scripts.
        */
        static const std::string TEXT_X_SCRIPT;

        /*!
         \brief The icon used for generic video file types.
        */
        static const std::string VIDEO_X_GENERIC;

        /*!
         \brief The icon used for generic address book file types.
        */
        static const std::string X_OFFICE_ADDRESS_BOOK;

        /*!
         \brief The icon used for generic calendar file types.
        */
        static const std::string X_OFFICE_CALENDAR;

        /*!
         \brief The icon used for generic document and letter file types.
        */
        static const std::string X_OFFICE_DOCUMENT;

        /*!
         \brief The icon used for generic presentation file types.
        */
        static const std::string X_OFFICE_PRESENTATION;

        /*!
         \brief The icon used for generic spreadsheet file types.
        */
        static const std::string X_OFFICE_SPREADSHEET;

    };

    /*!
      \brief Icons used to represent locations, either on the local filesystem, or through remote connections. Folders, trash, and workgroups are some examples.
     */
    class IconNamingPlaces {
    private:
        IconNamingPlaces() {};

    public:
        /*!
         \brief The standard folder icon used to represent directories on local filesystems, mail folders, and other hierarchical groups.
        */
        static const std::string FOLDER;

        /*!
         \brief The icon used for normal directories on a remote filesystem.
        */
        static const std::string FOLDER_REMOTE;

        /*!
         \brief The icon used for individual host machines under the "Network Servers" place in the file manager.
        */
        static const std::string NETWORK_SERVER;

        /*!
         \brief The icon for the "Network Servers" place in the desktop's file manager, and workgroups within the network.
        */
        static const std::string NETWORK_WORKGROUP;

        /*!
         \brief The icon used by the desktop's main menu for accessing places, applications, and other features.
        */
        static const std::string START_HERE;

        /*!
         \brief The icon for the user's special "Bookmarks" place.
        */
        static const std::string USER_BOOKMARKS;

        /*!
         \brief The icon for the special "Desktop" directory of the user.
        */
        static const std::string USER_DESKTOP;

        /*!
         \brief The icon for the special "Home" directory of the user.
        */
        static const std::string USER_HOME;

        /*!
         \brief The icon for the user's "Trash" place in the desktop's file manager.
        */
        static const std::string USER_TRASH;
    };

    /*!
      \brief Icons for presenting status to the user. This context contains icons for warning and error dialogs, as well as for the current weather,
            appointment alarms, and battery status.
     */
    class IconNamingStatus {
    private:
        IconNamingStatus() {};

    public:
        /*!
         \brief The icon used when an appointment was missed.
        */
        static const std::string APPOINTMENT_MISSED;

        /*!
         \brief The icon used when an appointment will occur soon.
        */
        static const std::string APPOINTMENT_SOON;

        /*!
         \brief The icon used to indicate high audio volume.
        */
        static const std::string AUDIO_VOLUME_HIGH;

        /*!
         \brief The icon used to indicate low audio volume.
        */
        static const std::string AUDIO_VOLUME_LOW;

        /*!
         \brief The icon used to indicate medium audio volume.
        */
        static const std::string AUDIO_VOLUME_MEDIUM;

        /*!
         \brief The icon used to indicate the muted state for audio playback.
        */
        static const std::string AUDIO_VOLUME_MUTED;

        /*!
         \brief The icon used when the battery is below 40%.
        */
        static const std::string BATTERY_CAUTION;

        /*!
         \brief The icon used when the battery is below 20%.
        */
        static const std::string BATTERY_LOW;

        /*!
         \brief The icon used when a dialog is opened to explain an error condition to the user.
        */
        static const std::string DIALOG_ERROR;

        /*!
         \brief The icon used when a dialog is opened to give information to the user that may be pertinent to the requested action.
        */
        static const std::string DIALOG_INFORMATION;

        /*!
         \brief The icon used when a dialog requesting the authentication credentials for a user is opened.
        */
        static const std::string DIALOG_PASSWORD;

        /*!
         \brief The icon used when a dialog is opened to ask a simple question of the user.
        */
        static const std::string DIALOG_QUESTION;

        /*!
         \brief The icon used when a dialog is opened to warn the user of impending issues with the requested action.
        */
        static const std::string DIALOG_WARNING;

        /*!
         \brief The icon used for a folder while an object is being dragged onto it, that is of a type that the directory can contain.
        */
        static const std::string FOLDER_DRAG_ACCEPT;

        /*!
         \brief The icon used for folders, while their contents are being displayed within the same window. This icon would normally be shown in a tree or list view, next to the main view of a folder's contents.
        */
        static const std::string FOLDER_OPEN;

        /*!
         \brief The icon used for folders, while their contents are being displayed in another window. This icon would typically be used when using multiple windows to navigate the hierarchy, such as in Nautilus's spatial mode.
        */
        static const std::string FOLDER_VISITING;

        /*!
         \brief The icon used when another image is being loaded, such as thumnails for larger images in the file manager.
        */
        static const std::string IMAGE_LOADING;

        /*!
         \brief The icon used when another image could not be loaded.
        */
        static const std::string IMAGE_MISSING;

        /*!
         \brief The icon used for an electronic mail that contains attachments.
        */
        static const std::string MAIL_ATTACHMENT;

        /*!
         \brief The icon used for an electronic mail that is unread.
        */
        static const std::string MAIL_UNREAD;

        /*!
         \brief The icon used for an electronic mail that is read.
        */
        static const std::string MAIL_READ;

        /*!
         \brief The icon used for an electronic mail that has been replied to.
        */
        static const std::string MAIL_REPLIED;

        /*!
         \brief The icon used for an electronic mail that contains a signature.
        */
        static const std::string MAIL_SIGNED;

        /*!
         \brief The icon used for an electronic mail that contains a signature which has also been verified by the security system.
        */
        static const std::string MAIL_SIGNED_VERIFIED;

        /*!
         \brief The icon for the repeat mode of a media player.
        */
        static const std::string MEDIA_PLAYLIST_REPEAT;

        /*!
         \brief The icon for the shuffle mode of a media player.
        */
        static const std::string MEDIA_PLAYLIST_SHUFFLE;

        /*!
         \brief The icon used when an error occurs trying to intialize the network connection of the computing device. This icon should be two computers, one in the background, with the screens of both computers, colored black, and with the theme's style element for errors, overlayed on top of the icon.
        */
        static const std::string NETWORK_ERROR;

        /*!
         \brief The icon used when no data is being transmitted or received, while the computing device is connected to a network. This icon should be two computers, one in the background, with the screens of both computers, colored black.
        */
        static const std::string NETWORK_IDLE;

        /*!
         \brief The icon used when the computing device is disconnected from the network. This icon should be a computer in the background, with a screen colored black, and the theme's icon element to show that a device is not accessible, in the foreground.
        */
        static const std::string NETWORK_OFFLINE;

        /*!
         \brief The icon used when data is being received, while the computing device is connected to a network. This icon should be two computers, one in the background, with its screen colored green, and the screen of the computer in the foreground, colored black.
        */
        static const std::string NETWORK_RECEIVE;

        /*!
         \brief The icon used when data is being transmitted, while the computing device is connected to a network. This icon should be two computers, one in the background, with its screen colored black, and the screen of the computer in the foreground, colored green.
        */
        static const std::string NETWORK_TRANSMIT;

        /*!
         \brief The icon used data is being both transmitted and received simultaneously, while the computing device is connected to a network. This icon should be two computers, one in the background, with the screens of both computers, colored green.
        */
        static const std::string NETWORK_TRANSMIT_RECEIVE;

        /*!
         \brief The icon used when an error occurs while attempting to print. This icon should be the theme's printer device icon, with the theme's style element for errors, overlayed on top of the icon.
        */
        static const std::string PRINTER_ERROR;

        /*!
         \brief The icon used while a print job is successfully being spooled to a printing device. This icon should be the theme's printer device icon, with a document emerging from the printing device.
        */
        static const std::string PRINTER_PRINTING;

        /*!
         \brief The icon used to indicate that the security level of a connection is known to be secure, using strong encryption and a valid certificate.
        */
        static const std::string SECURITY_HIGH;

        /*!
         \brief The icon used to indicate that the security level of a connection is presumed to be secure, using strong encryption, and a certificate that could not be automatically verified, but which the user has chosen to trust.
        */
        static const std::string SECURITY_MEDIUM;

        /*!
         \brief The icon used to indicate that the security level of a connection is presumed to be insecure, either by using weak encryption, or by using a certificate that the could not be automatically verified, and which the user has not chosent to trust.
        */
        static const std::string SECURITY_LOW;

        /*!
         \brief The icon used when an update is available for software installed on the computing device, through the system software update program.
        */
        static const std::string SOFTWARE_UPDATE_AVAILABLE;

        /*!
         \brief The icon used when an urgent update is available through the system software update program.
        */
        static const std::string SOFTWARE_UPDATE_URGENT;

        /*!
         \brief The icon used when an error occurs while attempting to synchronize data from the computing device, to another device.
        */
        static const std::string SYNC_ERROR;

        /*!
         \brief The icon used while data is successfully synchronizing to another device.
        */
        static const std::string SYNC_SYNCHRONIZING;

        /*!
         \brief The icon used when a task is due soon.
        */
        static const std::string TASK_DUE;

        /*!
         \brief The icon used when a task that was due, has been left incomplete.
        */
        static const std::string TASK_PAST_DUE;

        /*!
         \brief The icon used when a user on a chat network is available to initiate a conversation with.
        */
        static const std::string USER_AVAILABLE;

        /*!
         \brief The icon used when a user on a chat network is away from their keyboard and the chat program.
        */
        static const std::string USER_AWAY;

        /*!
         \brief The icon used when a user on a chat network has not been an active participant in any chats on the network, for an extended period of time.
        */
        static const std::string USER_IDLE;

        /*!
         \brief The icon used when a user on a chat network is not available.
        */
        static const std::string USER_OFFLINE;

        /*!
         \brief The icon for the user's "Trash" in the desktop's file manager, when there are items in the "Trash" waiting for disposal or recovery.
        */
        static const std::string USER_TRASH_FULL;

        /*!
         \brief The icon used while the weather for a region is "clear skies".
        */
        static const std::string WEATHER_CLEAR;

        /*!
         \brief The icon used while the weather for a region is "clear skies" during the night.
        */
        static const std::string WEATHER_CLEAR_NIGHT;

        /*!
         \brief The icon used while the weather for a region is "partly cloudy".
        */
        static const std::string WEATHER_FEW_CLOUDS;

        /*!
         \brief The icon used while the weather for a region is "partly cloudy" during the night.
        */
        static const std::string WEATHER_FEW_CLOUDS_NIGHT;

        /*!
         \brief The icon used while the weather for a region is "foggy".
        */
        static const std::string WEATHER_FOG;

        /*!
         \brief The icon used while the weather for a region is "overcast".
        */
        static const std::string WEATHER_OVERCAST;

        /*!
         \brief The icon used while a sever weather alert is in effect for a region.
        */
        static const std::string WEATHER_SEVERE_ALERT;

        /*!
         \brief The icon used while rain showers are occurring in a region.
        */
        static const std::string WEATHER_SHOWERS;

        /*!
         \brief The icon used while scattered rain showers are occurring in a region.
        */
        static const std::string WEATHER_SHOWERS_SCATTERED;

        /*!
         \brief The icon used while snow showers are occurring in a region.
        */
        static const std::string WEATHER_SNOW;

        /*!
         \brief The icon used while storms are occurring in a region.
        */
        static const std::string WEATHER_STORM;

    };
}

#endif // ICONNAMING_H
