/*
  Copyright (C) 2010 Yvon Tanguy

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<sys/types.h>
#include<sys/stat.h>
#include<dirent.h>
#include<unistd.h>
#include<errno.h>
#include<string.h>
#include<limits>
#include<iostream>
#include<algorithm>
#include<stdexcept>

#ifndef TIXML_USE_STL
#define TIXML_USE_STL
#endif

#include "tinyxml/tinyxml.h"
#include "icontheme.h"
#include "fdexception.h"
#include "xdgbasedirectory.h"
#include "inistylefile.h"

using namespace std;
using namespace freedesktop;

/* static and constant initialisations: */

const string IconTheme::ICONS_THEMES_SPEC_SUBDIRECTORY = "icons";
const string IconTheme::THEME_FILE = "index.theme";
const string IconTheme::DEFAULT_THEME = "hicolor";

// Theme file content
static const string THEME_START_SECTION = "Icon Theme";
static const string THEME_KEY_NAME = "Name";
static const string THEME_KEY_COMMENT = "Comment";
static const string THEME_KEY_INHERITS = "Inherits";
static const string THEME_KEY_DIRECTORIES = "Directories";
static const string THEME_KEY_HIDDEN = "Hidden";
static const string THEME_KEY_EXAMPLE = "Example";

static const string THEME_START_SECTION_DIRECTORY = "$DIRECTORY";
static const string THEME_KEY_DIRECTORY_SIZE = "Size";
static const string THEME_KEY_DIRECTORY_CONTEXT = "Context";
static const string THEME_KEY_DIRECTORY_TYPE = "Type";
static const string THEME_KEY_DIRECTORY_MAXSIZE = "MaxSize";
static const string THEME_KEY_DIRECTORY_MINSIZE = "MinSize";
static const string THEME_KEY_DIRECTORY_THRESHOLD = "Threshold";

static const string THEME_KEY_DIRECTORY_TYPE_FIXED = "Fixed";
static const string THEME_KEY_DIRECTORY_TYPE_SCALABLE = "Scalable";
static const string THEME_KEY_DIRECTORY_TYPE_THRESHOLD = "Threshold";

static const std::string ICON_PNG_EXT = ".png";
static const std::string ICON_SVG_EXT = ".svg";
static const std::string ICON_XPM_EXT = ".xpm";

int IconTheme::ICONS_FILES_FORMATS = 0;

// internal use: deallocate buffers create at runtime (themesList)
IconTheme IconTheme::staticInstance;
stringList *IconTheme::iconsExtensions = NULL;
IconTheme *IconTheme::currentTheme = NULL;
HMapThemes *IconTheme::themesList = NULL;

/* methods definitions */

IconTheme::IconTheme() {
  flagStaticInstance = true;
  this->baseDirectories.clear();
  this->isLoaded = true;
  this->listThemeDirectories = NULL;
  setIconsFilesFormats(ICON_FILE_FORMAT_XPM | ICON_FILE_FORMAT_PNG);
}

IconTheme::IconTheme(string &_intName) {
  flagStaticInstance = false;

  this->baseDirectories.clear();
  this->internalName = _intName;
  this->isLoaded = false;
  this->listThemeDirectories = NULL;
}

IconTheme::~IconTheme() {
  if (flagStaticInstance) {
    cleanCache();
  } else {

    if (NULL != this->listThemeDirectories) {
      while (!this->listThemeDirectories->empty()) {
	ListThemeDirectories::iterator itFirst = this->listThemeDirectories->begin();
	delete *itFirst;
	this->listThemeDirectories->erase(itFirst);
      }
      delete this->listThemeDirectories;
      this->listThemeDirectories = NULL;
    }
    this->isLoaded = false;
    this->baseDirectories.clear();
  }
}

const std::string &IconTheme::GetIconsThemesSpecDirectories() {
  static const string IconsThemesSpecDirectories = "$HOME/.icons:"
    + XDGBaseDirectory::GetEnvXDGDataDirs() + ":"
    + XDGBaseDirectory::GetEnvXDGDataHome() + ":"
    + "/usr/share/pixmaps";

  return IconsThemesSpecDirectories;
}

const stringList &IconTheme::GetIconsThemesDirectories() {
  static const stringList IconsThemesDirectories = XDGBaseDirectory::GetXDGDirs(IconTheme::GetIconsThemesSpecDirectories(), IconTheme::ICONS_THEMES_SPEC_SUBDIRECTORY);

  return IconsThemesDirectories;
}

void IconTheme::cleanCache() {
  if (NULL != themesList) {
    while (!themesList->empty()) {
      HMapThemes::iterator itFirst = themesList->begin();
      delete itFirst->second;
      themesList->erase(itFirst);

    }
    delete themesList;
    themesList = NULL;
  }

  if (NULL != iconsExtensions) {
    iconsExtensions->clear();
    delete iconsExtensions;
    iconsExtensions = NULL;
  }
}

int IconTheme::getIconsFilesFormats() {
  return IconTheme::ICONS_FILES_FORMATS;
}

void IconTheme::setIconsFilesFormats(int formats) {
  int newFormat = 0;

  if ((formats & (ICON_FILE_FORMAT_XPM | ICON_FILE_FORMAT_PNG | ICON_FILE_FORMAT_SVG)) == formats) {
    if (NULL == iconsExtensions) {
      iconsExtensions = new stringList(3);
    } else {
      iconsExtensions->clear();
    }

    if (formats & ICON_FILE_FORMAT_PNG) {
      newFormat |= ICON_FILE_FORMAT_PNG;
      iconsExtensions->push_back(ICON_PNG_EXT);
    }

    if (formats & ICON_FILE_FORMAT_SVG) {
      newFormat |= ICON_FILE_FORMAT_SVG;
      iconsExtensions->push_back(ICON_SVG_EXT);
    }

    if (formats & ICON_FILE_FORMAT_XPM) {
      newFormat = ICON_FILE_FORMAT_XPM;
      iconsExtensions->push_back(ICON_XPM_EXT);
    }

    IconTheme::ICONS_FILES_FORMATS = newFormat;

  } else {

    throw FD_EXCEPTION("Bad file format");

  }
}

#define ENV_DESKTOP_SESSION "DESKTOP_SESSION"
#define ENV_KDE_FULL_SESSION "KDE_FULL_SESSION"
#define DESKTOP_SESSION_GNOME "gnome"
#define DESKTOP_SESSION_KDE "kde"
#define DESKTOP_SESSION_KDE_PLASMA "kde-plasma" // KDE 4.6 Ubuntu 11.04
#define DESKTOP_SESSION_DEFAULT "default"
#define DESKTOP_CONF_GNOME "$HOME/.gconf/desktop/gnome/interface/%gconf.xml"
#define DESKTOP_CONF_KDE "$HOME/.kde/share/config/kdeglobals"
#define DESKTOP_THEME_KDE "default.kde"
#define DESKTOP_THEME_KDE4 "default.kde4"
#define DESKTOP_THEME_GNOME "gnome"

void IconTheme::buildVirtualTheme() {
  // Quick an dirty virtual theme with all avaible directories

  staticInstance.internalName = "virtual";
  staticInstance.isLoaded = true;
  staticInstance.userName = "Virtual Theme";
  staticInstance.hidden = true;
  //staticInstance.listThemeDirectories = new ListThemeDirectories();

  cout << "Building virtual theme..." << endl;

  initThemesList();

  for (HMapThemes::iterator itThemesList = themesList->begin(); itThemesList != themesList->end(); ++itThemesList) {
    // ignore the hiclor theme, it will be include on others themes
    if ((*itThemesList).first != DEFAULT_THEME) {
      staticInstance.inherits.push_back((*itThemesList).first);
    }
  }
  
  currentTheme = &staticInstance;
}

IconTheme *IconTheme::getGnomeTheme() {
  IconTheme *theme = NULL;

  // Try to load the gconf file, with the icon theme inside:
  tinyxml::TiXmlDocument xmlDoc(XDGBaseDirectory::EnvPathToRealPath(DESKTOP_CONF_GNOME));
  if (xmlDoc.LoadFile()) {
    tinyxml::TiXmlHandle hDoc(&xmlDoc);
    tinyxml::TiXmlElement *pElem;
    tinyxml::TiXmlHandle hRoot(0);
    
    pElem = hDoc.FirstChildElement("gconf").Element();
    if (pElem) {
      pElem = pElem->FirstChildElement("entry");
      for (; pElem != NULL && currentTheme == NULL; pElem = pElem->NextSiblingElement()) {
	if (!strcmp(pElem->Attribute("name"), "icon_theme")) {
	  tinyxml::TiXmlElement *pStrElem = pElem->FirstChildElement("stringvalue");
	  if (NULL != pStrElem) {
	    theme = findTheme(pStrElem->GetText());
	  }
	}
      }
    }
    
  } else {
    cerr << "can't load gnome config file: " << DESKTOP_CONF_GNOME << endl;
  }

  // Theme not found, used the default "gnome" theme
  if (NULL == theme) {
    cout << "use default gnome theme" << endl;
    currentTheme = findTheme(DESKTOP_THEME_GNOME);
  }

  return theme;
}

IconTheme *IconTheme::getCurrentTheme() {
  if (NULL == currentTheme) {
    char *envValue = NULL;
    envValue = getenv(ENV_DESKTOP_SESSION);
    if (NULL != envValue) {
      if (!strcmp(DESKTOP_SESSION_GNOME, envValue)) {
	currentTheme = getGnomeTheme();
	
      } else if (!strcmp(DESKTOP_SESSION_KDE, envValue) || !strcmp(DESKTOP_SESSION_KDE_PLASMA, envValue)) {
	// TODO: Load the KDE config
	
	// Theme not found, used the default "kde4" theme
	currentTheme = findTheme(DESKTOP_THEME_KDE4);
	if (NULL == currentTheme) {
	  // Theme not found, used the default "kde" theme
	  currentTheme = findTheme(DESKTOP_THEME_KDE);
	}
      } else if (!strcmp(DESKTOP_SESSION_DEFAULT, envValue)) {
	// FIXME
	currentTheme = findTheme(DESKTOP_THEME_KDE4);
	if (NULL == currentTheme) {
	  // Theme not found, used the default "kde" theme
	  currentTheme = findTheme(DESKTOP_THEME_KDE);
	}
      }
    }
  }

  // No desktop in use found
  // Build a virtual theme
  if (NULL == currentTheme) {
    cout << "Unknown dektop in use" << endl;
    buildVirtualTheme();
  }
  

  if (NULL == currentTheme) {
    currentTheme = findTheme(DEFAULT_THEME);
  }
  return currentTheme;
}

void IconTheme::setCurrentTheme(IconTheme *theme) throw(FDException) {
  if (NULL == theme) {
    throw FD_EXCEPTION("Theme is null");
  }
  try {
    theme->loadTheme();
  } catch (FDException e) {
    e.addTrace(__FILE__, __LINE__, __func__);
    throw e;
  }
  currentTheme = theme;
}

IconTheme *IconTheme::findTheme(const std::string &name) {
  IconTheme *theme = NULL;

  if (!name.empty()) {
    if (NULL == themesList) {
      initThemesList();
    }

    HMapThemes::iterator itFind = themesList->find(name);
    if (itFind != themesList->end()) {
      theme = itFind->second;
    }
  }

  if (NULL == theme) {
    cout << "Theme not found: " << name << endl;
  }

  return theme;
}

void IconTheme::initThemesList() {
  DIR *dir = NULL;
  struct dirent *dp = NULL;
  struct stat statDir;
  //struct stat statTheme;
  IconTheme *theme = NULL;

  if (NULL != themesList) {
    return;
  }

  themesList = new map<string, IconTheme *>();

  // Loop on the directories icons themes
  for (string::size_type i=0; i<GetIconsThemesDirectories().size(); i++) {
    dir = opendir(GetIconsThemesDirectories().at(i).c_str());
    // ignore inexistant directories
    if (NULL != dir) {
      dp = readdir(dir);
      while (NULL != dp) {
	string intName = dp->d_name;

	string path = GetIconsThemesDirectories().at(i) + "/" + intName;
	if (intName != "." && intName != ".." && stat(path.c_str(), &statDir) == 0 && S_ISDIR(statDir.st_mode)) {
	  HMapThemes::iterator itFind = themesList->find(intName);
	  if (itFind == themesList->end()) {
	    theme = new IconTheme(intName);
	    (*themesList)[intName] = theme;
	  } else {
	    theme = itFind->second;
	  }
	  theme->baseDirectories.push_back(path);

	  string themeFilename = path + "/" + THEME_FILE;
	  if (theme->themeFilename.empty() && fileExists(themeFilename)) {
	    theme->themeFilename.swap(themeFilename);
	  }
	}
	// Check the next entry:
	dp = readdir(dir);
      }
      (void) closedir(dir);
    }
  }
}

const string *IconTheme::getLocaleValue(const IniKeysValues *themeSection, const std::string *_key) const {
  IniKeysValues::const_iterator it;

  // full locale value first
  it = themeSection->find(*_key + "[" + getLocale() + "]");
  if (it != themeSection->end()) {
    return &((*it).second);
  }

  // if not found, search for just the lang part
  it = themeSection->find(*_key + "[" + getLocaleLang() + "]");
  if (it != themeSection->end()) {
    return &((*it).second);
  }

  // No local value, fallback to the default
  it = themeSection->find(*_key);
  if (it != themeSection->end()) {
    return &((*it).second);
  }

  return NULL;
}

void IconTheme::loadTheme() throw(FDException) {
  if (this->isLoaded) {
    return;
  }
    
  // prevent from loading it again
  this->isLoaded = true;

  if (this->themeFilename.empty()) {
    this->hidden = true;
    throw FD_EXCEPTION("No filename to read for this theme");
  }

  cout << "Load icon theme " + this->themeFilename << endl;

  IniStyleFile iniFile(this->themeFilename);
  try {
    iniFile.readFile();
  } catch (FDException e) {
    this->isLoaded = false;
    e.addTrace(__FILE__, __LINE__, __func__);
    throw e;
  }
    
  IniKeysValues *themeSection = NULL;
  IniSections::iterator itTheme = iniFile.getIniSections()->find(THEME_START_SECTION);
  if (itTheme == iniFile.getIniSections()->end()) {
    throw FD_EXCEPTION("Empty theme file (no \"Icon Theme\" section).");
  }
  themeSection = (*itTheme).second;

  // User name:
  const string *readValue = getLocaleValue(themeSection, &THEME_KEY_NAME);
  if (NULL == readValue) {
    this->hidden = true;
    // No theme name at all, but it is mandatory:
    throw FD_EXCEPTION("Theme name is required for theme in " + this->themeFilename + ".");
  }
  this->userName = *readValue;

  // Locale comment:
  readValue = getLocaleValue(themeSection, &THEME_KEY_COMMENT);
  if (NULL == readValue) {
    this->hidden = true;
    // No theme comment at all, but it is mandatory:
    throw FD_EXCEPTION("Theme comment is required for theme " + this->internalName + ".");
  }
  this->comment = *readValue;

  // Inherits themes
  IniKeysValues::const_iterator it;
  it = themeSection->find(THEME_KEY_INHERITS);
  if (it != themeSection->end()) {
    // Add the default "hicolor" theme if not present
    string inheritsLine = (*it).second;
    // replace space char found in some themes
    stringReplace(inheritsLine, " ", "");
    stringSplit(inheritsLine, this->inherits, ",");
    if (this->internalName != DEFAULT_THEME && !stringListContains(this->inherits, DEFAULT_THEME)) {
      this->inherits.push_back(DEFAULT_THEME);
    }
      
    //printList(this->inherits);
  } else {
    this->inherits.clear();
    if (this->internalName != DEFAULT_THEME)
      this->inherits.push_back(DEFAULT_THEME);
    //    cout << "inherits of " << this->internalName << endl;
  }
    
  // Directories
  it = themeSection->find(THEME_KEY_DIRECTORIES);
  if (it != themeSection->end()) {
    stringSplit((*it).second, this->directories, ",");
  } else {
    this->hidden = true;
    throw FD_EXCEPTION("Theme directories are requiered for theme " + this->internalName + " in \"" + this->themeFilename + "\".");
  }

  // Hidden
  this->hidden = false; // Default value
  it = themeSection->find(THEME_KEY_HIDDEN);
  if (it != themeSection->end()) {
    string boolValueStr = (*it).second;
    stringToLower(boolValueStr);
    if (boolValueStr == "true") {
      this->hidden = true;
    }
  }

  // Example
  it = themeSection->find(THEME_KEY_EXAMPLE);
  if (it != themeSection->end()) {
    this->example = (*it).second;
  } else {
    this->example = "";
  }
  themeSection = NULL;

  // Get directories sections
  // By copy
  this->listThemeDirectories = new ListThemeDirectories();
  for (itTheme = iniFile.getIniSections()->begin(); itTheme != iniFile.getIniSections()->end(); itTheme++) {
    if (THEME_START_SECTION != (*itTheme).first) {
      IconThemeDirectorySection *dirSection = new IconThemeDirectorySection((*itTheme).first, (*itTheme).second, this);
      this->listThemeDirectories->push_back(dirSection);
    }
  }

  this->checkThemeProblems();

}

void IconTheme::checkThemeProblems() {
  //    cout << "checking theme " << this->internalName << endl;
  if (this->isLoaded) {
    // First pass: check if all themes in this->directories exists in this->directoriesSections
    // TODO

    // Second pass: check if all this->directoriesSections exists in this->directories
    // TODO
  }
  //    cout << "checking theme " << this->internalName << ": done" << endl;
}

/* freedesktop's icons lookup functions */

string IconTheme::findIcon(const string &icon, int size) throw(FDException) {
  IconTheme *theme = getCurrentTheme();
  if (NULL == theme) {
    throw FD_EXCEPTION("No current theme defined");
  }

  string filename;
  try {
    filename = findIconHelper(icon, size, theme);

    if (filename.empty()) {
      filename = lookupFallbackIcon(icon);
    }
  } catch (FDException e) {
    e.addTrace(__FILE__, __LINE__, __func__);
    throw e;
  }

  if (filename.empty()) {
    cerr << "no icon found for " << icon << " in theme " << theme->getInternalName() << endl;
  }

  return filename;
}

string IconTheme::findIconHelper(const string &icon, int size, IconTheme *theme) throw(FDException) {
  if (NULL == theme) {
    throw FD_EXCEPTION("The theme is null");
  }
  if (icon.empty()) {
    throw FD_EXCEPTION("Error in icon or size parameter: empty");
  }

  // Try to load the theme
  try {
    theme->loadTheme();
  } catch (FDException e) {
    e.addTrace(__FILE__, __LINE__, __func__);
    throw e;
  }

  //cout << __func__ << "(" << icon << "," << theme->internalName << ")" << endl;

  // Find the icon in this theme:
  string filename = "";
  try {
    filename = lookupIcon(icon, size, theme);
  } catch (FDException e) {
    e.addTrace(__FILE__, __LINE__, __func__);
    throw e;
  }


  // if not found, and if the theme has parents, search in parents
  if (filename.empty() && !theme->inherits.empty()) {
    stringList::size_type sizeList = theme->inherits.size();
    stringList::size_type countList = 0;
    while (countList < sizeList && filename.empty()) {
      try {
	filename = findIconHelper(icon, size, findTheme(theme->inherits.at(countList)));
	countList++;
      } catch (FDException e) {
	e.addTrace(__FILE__, __LINE__, __func__);
	throw e;
      }
    }
  }


  return filename;
}

string IconTheme::lookupIcon(const string &iconname, int size, IconTheme *theme) {
  // no check for input param, already done in public functions.

  /*
    for each subdir in $(theme subdir list) {
    for each directory in $(basename list) {
    for extension in ("png", "svg", "xpm") {
    if DirectoryMatchesSize(subdir, size) {
    filename = directory/$(themename)/subdir/iconname.extension
    if exist filename
    return filename
    }
    }
    }
    }
  */

  //    cout << __func__ << "(" << iconname << "," << theme->internalName << ")" << endl;
  string closest_filename = "";
  ListThemeDirectories *listDirs = theme->listThemeDirectories;

  if (listDirs == NULL) {
    return closest_filename;
  }

  try {
    for (ListThemeDirectories::size_type itSubDir = 0; itSubDir < listDirs->size(); itSubDir++) {
      //            cout << "itSubDir=" << itSubDir << " max=" << listDirs->size() << endl;
      for (stringList::size_type itThemeBaseDir = 0; itThemeBaseDir < theme->baseDirectories.size(); itThemeBaseDir++) {
	//                cout << "\titThemeBaseDir=" << itThemeBaseDir << " max=" << theme->baseDirectories.size() << endl;
	for (stringList::size_type itIconExt = 0; itIconExt < iconsExtensions->size(); itIconExt++) {
	  //                    cout << "\t\titIconExt=" << itIconExt << " max=" << iconsExtensions.size() << endl;
	  if (listDirs->at(itSubDir)->directoryMatchesSize(size)) {
	    string filename = theme->baseDirectories.at(itThemeBaseDir) + DIR_SEPARATOR_CHAR +
	      listDirs->at(itSubDir)->name + DIR_SEPARATOR_CHAR +
	      iconname + iconsExtensions->at(itIconExt);
	    if (fileExists(filename)) {
	      return filename;
	    }

	  }
	}
      }
    }
  }  catch (out_of_range ofr) {
    printList(theme->baseDirectories);
    printList(*iconsExtensions);
    throw FD_EXCEPTION(ofr.what());
  } catch (FDException e) {
    string param = string(__func__) + "(" + theme->themeFilename + ")";
    e.addTrace(__FILE__, __LINE__, param.c_str());
    throw e;
  }
  
  /*
    minimal_size = MAXINT
    for each subdir in $(theme subdir list) {
    for each directory in $(basename list) {
    for extension in ("png", "svg", "xpm") {
    filename = directory/$(themename)/subdir/iconname.extension
    if exist filename and DirectorySizeDistance(subdir, size) < minimal_size {
    closest_filename = filename
    minimal_size = DirectorySizeDistance(subdir, size)
    }
    }
    }
    }
    if closest_filename set
    return closest_filename
    return none
  */

  int minimal_size = numeric_limits<int>::max();
    
  try {
    for (ListThemeDirectories::size_type itSubDir = 0; itSubDir < listDirs->size(); itSubDir++) {
      for (stringList::size_type itThemeBaseDir = 0; itThemeBaseDir < theme->baseDirectories.size(); itThemeBaseDir++) {
	for (stringList::size_type itIconExt = 0; itIconExt < iconsExtensions->size(); itIconExt++) {
	  string filename = theme->baseDirectories.at(itThemeBaseDir) + DIR_SEPARATOR_CHAR +
	    listDirs->at(itSubDir)->name + DIR_SEPARATOR_CHAR +
	    iconname + iconsExtensions->at(itIconExt);
	  int dist = listDirs->at(itSubDir)->directorySizeDistance(size);
	  if (dist < minimal_size && fileExists(filename)) {
	    closest_filename = filename;
	    minimal_size = dist;
	  }
	}
      }
    }
  } catch (out_of_range oor) {
    throw FD_EXCEPTION(oor.what());
  } catch (FDException e) {
    e.addTrace(__FILE__, __LINE__, __func__);
    throw e;
  }
  
  return closest_filename;
}

string IconTheme::lookupFallbackIcon(const string &iconname) {
  /*
    for each directory in $(basename list) {
    for extension in ("png", "svg", "xpm") {
    if exists directory/iconname.extension
    return directory/iconname.extension
    }
    }
    return none
  */

  //    cout << __func__ << "(" << iconname << ")" << endl;
  for (string::size_type i=0; i<GetIconsThemesDirectories().size(); i++) {
    for (stringList::size_type itIconExt = 0; itIconExt < iconsExtensions->size(); itIconExt++) {
      string filename = GetIconsThemesDirectories().at(i) + DIR_SEPARATOR_CHAR +
	iconname + iconsExtensions->at(itIconExt);
      if (fileExists(filename)) {
	return filename;
      }
    }
  }
  return "";
}

IconThemeDirectorySection::IconThemeDirectorySection(const std::string _name, const IniKeysValues *keyVals, IconTheme *_parent) {
  this->name = _name;
  this->type = "";
  this->parent = _parent;
  this->size = 0;
  this->minSize = 0;
  this->maxSize = 0;
  this->threshold = 2;

  try {
    initVals(keyVals);
  } catch (FDException e) {
    e.addTrace(__FILE__, __LINE__, __func__);
    throw e;
  }
}

IconThemeDirectorySection::~IconThemeDirectorySection() {
  //    cout << __func__ << " (" << this->name << ", " << this->parent->getInternalName() << ")" << endl;
  this->name.clear();
  this->type.clear();
  this->parent = NULL;
}

void IconThemeDirectorySection::initVals(const IniKeysValues *keyVals) throw (FDException) {
  try {
    // Required
    this->size = atoi(keyVals->at(THEME_KEY_DIRECTORY_SIZE).c_str());

    // Not required
    if (keyVals->find(THEME_KEY_DIRECTORY_TYPE) != keyVals->end()) {
      this->type = keyVals->at(THEME_KEY_DIRECTORY_TYPE);
    } else {
      this->type = THEME_KEY_DIRECTORY_TYPE_THRESHOLD;
    }

    if (keyVals->find(THEME_KEY_DIRECTORY_MINSIZE) != keyVals->end()) {
      this->minSize = atoi(keyVals->at(THEME_KEY_DIRECTORY_MINSIZE).c_str());
    } else {
      this->minSize = size;
    }

    if (keyVals->find(THEME_KEY_DIRECTORY_MAXSIZE) != keyVals->end()) {
      this->maxSize = atoi(keyVals->at(THEME_KEY_DIRECTORY_MAXSIZE).c_str());
    } else {
      this->maxSize = size;
    }

    if (keyVals->find(THEME_KEY_DIRECTORY_THRESHOLD) != keyVals->end()) {
      this->threshold = atoi(keyVals->at(THEME_KEY_DIRECTORY_THRESHOLD).c_str());
    } else {
      this->threshold = 2;
    }
  } catch (out_of_range oor) {
    throw FD_EXCEPTION(oor.what());
  } catch (...) {
    throw FD_EXCEPTION("error in init of " + this->name + " for " + this->parent->getInternalName());
  }
}


bool IconThemeDirectorySection::directoryMatchesSize(const int &iconSize) const {
  /*
    if Type is Fixed
    return Size == iconsize
    if Type is Scaled
    return MinSize <= iconsize <= MaxSize
    if Type is Threshold
    return Size - Threshold <= iconsize <= Size + Threshold
  */

  bool match = false;
  if (THEME_KEY_DIRECTORY_TYPE_FIXED == this->type) {
    match = (this->size == iconSize);
  } else if (THEME_KEY_DIRECTORY_TYPE_SCALABLE == this->type) {
    match = (this->minSize <= iconSize && iconSize <= this->maxSize);
  } else if (THEME_KEY_DIRECTORY_TYPE_THRESHOLD == this->type) {
    match = (((this->size - this->threshold) <= iconSize) && (iconSize <= (this->size + this->threshold)));
  }

  return match;

}

int IconThemeDirectorySection::directorySizeDistance(const int &iconSize) const {
  /*
    if Type is Fixed
    return abs(Size - iconsize)
    if Type is Scaled
    if iconsize < MinSize
    return MinSize - iconsize
    if iconsize > MaxSize
    return iconsize - MaxSize
    return 0
    if Type is Threshold
    if iconsize < Size - Threshold
    return MinSize - iconsize
    if iconsize > Size + Threshold
    return iconsize - MaxSize
    return 0
  */

  int distance = 0;
  if (THEME_KEY_DIRECTORY_TYPE_FIXED == this->type) {
    distance = abs(this->size - iconSize);
  } else if (THEME_KEY_DIRECTORY_TYPE_SCALABLE == this->type) {
    if (iconSize < this->minSize)
      distance = this->minSize - iconSize;
    else if (iconSize > this->maxSize)
      distance = iconSize - this->maxSize;
  } else if (THEME_KEY_DIRECTORY_TYPE_THRESHOLD == this->type) {
    if (iconSize < (size - threshold))
      distance = minSize - iconSize;
    else if (iconSize > (size + threshold))
      distance = iconSize - maxSize;
  }

  return distance;
}

std::string IconTheme::findBestIcon(stringList &iconList, int size) throw (FDException) {
  IconTheme *theme = getCurrentTheme();
  if (NULL == theme) {
    throw FD_EXCEPTION("No current theme defined");
  }

  try {
    string filename = findBestIconHelper(iconList, size, theme);
    stringList::size_type sizeList = iconList.size();
    stringList::size_type countList = 0;
    while (countList < sizeList && filename.empty()) {
      filename = lookupFallbackIcon(iconList.at(countList));
      countList++;
    }

    return filename;
  } catch (FDException e) {
    e.addTrace(__FILE__, __LINE__, __func__);
    throw e;
  }
}

std::string IconTheme::findBestIconHelper(stringList &iconList, int size, IconTheme *theme) throw (FDException) {

  if (NULL == theme) {
    throw FD_EXCEPTION("The theme is null");
  }
  if (iconList.empty()) {
    throw FD_EXCEPTION("Error: icon list parameter: empty");
  }

  // Try to load the theme
  try {
    theme->loadTheme();
  } catch (FDException e) {
    e.addTrace(__FILE__, __LINE__, __func__);
    throw e;
  }

  // Find the icon in the current theme:
  string filename = "";
  stringList::size_type sizeList = iconList.size();
  stringList::size_type countList = 0;
  while (countList < sizeList && filename.empty()) {
    try {
      filename = lookupIcon(iconList.at(countList), size, theme);
    } catch (FDException e) {
      e.addTrace(__FILE__, __LINE__, __func__);
      throw e;
    }
  }

  // If not found, find the icon in the inherits themes:
  if (filename.empty() && !theme->inherits.empty()) {
    sizeList = theme->inherits.size();
    countList = 0;
    while (countList < sizeList && filename.empty()) {
      try {
	filename = findBestIconHelper(iconList, size, findTheme(theme->inherits.at(countList)));
      } catch (FDException e) {
	e.addTrace(__FILE__, __LINE__, __func__);
	throw e;
      }
      countList++;
    }
  }

  return filename;
}
