/*
    Copyright (C) 2010 Yvon Tanguy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef ICONTHEME_H
#define ICONTHEME_H

#include<string>
#include<map>
#include "libfreedesktop.h"
#include "fdexception.h"
#include "inistylefile.h"

namespace freedesktop {

#define ICON_FILE_FORMAT_XPM 0x01
#define ICON_FILE_FORMAT_PNG 0x02
#define ICON_FILE_FORMAT_SVG 0x04

    class IconTheme;
    class IconThemeDirectorySection;
    typedef std::map<std::string, IconTheme *> HMapThemes;
    typedef std::vector<IconThemeDirectorySection *> ListThemeDirectories;

    class IconThemeDirectorySection {
        friend class IconTheme;
    public:
        ~IconThemeDirectorySection();

        bool directoryMatchesSize(const int &iconSize) const;
        int directorySizeDistance(const int &iconSize) const;

    private:
        IconThemeDirectorySection(const std::string _name, const IniKeysValues *keyVals, IconTheme *_parent);

        void initVals(const IniKeysValues *keyVals) throw (FDException);

        std::string name;
        std::string type;
        int size;
        int minSize;
        int maxSize;
        int threshold;
        IconTheme *parent;
    };

    /*!
    \class IconTheme icontheme.h
    \brief An icon theme is a set of icons that share a common look and feel.
    \par Overview
        An icon theme is a set of icons that share a common look and feel. The user can then select the icon theme that they want to use,
        and all apps use icons from the theme. The initial user of icon themes is the icon field of the desktop file specification,
        but in the future it can have other uses (such as mimetype icons).

        From a programmer perspective an icon theme is just a mapping. Given a set of directories to look for icons in and a theme name
        it maps from icon name and nominal icon size to an icon filename.

    \par Definitions
        \li Icon Theme: An icon theme is a named set of icons. It is used to map from an iconname and size to a file. Themes may inherit from other themes as a way to extend them.
        \li Icon file: An icon file is an image that can be loaded and used as an icon. The supported image file formats are PNG, XPM and SVG. PNG is the recommended bitmap format,
            and SVG is for vectorized icons. XPM is supported due to backwards compability reasons, and it is not recommended that new themes use XPM files.
            Support for SVGs is optional.
        \li Base Directory: Icons and themes are searched for in a set of directories, called base directories. The themes are stored in subdirectories of the base directories.

    \par For more informations
        <a href="http://standards.freedesktop.org/icon-theme-spec/icon-theme-spec-0.11.html">freedesktop specifications</a>
    \example test_icontheme.cpp
     */
    class IconTheme {
    public:
        IconTheme(std::string &_intName);
        ~IconTheme();

        /*!
          The supported image file formats are PNG, XPM and SVG.
          \return the icons files format wich will be fetch
          \see ICON_FILE_FORMAT_XPM
          \see ICON_FILE_FORMAT_PNG
          \see ICON_FILE_FORMAT_SVG
          */
        static int getIconsFilesFormats();

        /*!
          The supported image file formats are PNG, XPM and SVG.
          \see ICON_FILE_FORMAT_XPM
          \see ICON_FILE_FORMAT_PNG
          \see ICON_FILE_FORMAT_SVG
          \param formats: ICON_FILE_FORMAT_XPM | ICON_FILE_FORMAT_PNG | ICON_FILE_FORMAT_SVG
          \attention if the files formats set is not recognize, the value is not change.
          \attention the default value is: ICON_FILE_FORMAT_XPM | ICON_FILE_FORMAT_PNG
          */
        static void setIconsFilesFormats(int formats);

        /**
          \brief Get the current theme.
          \note This function search for the current desktop theme, if the internal currentTheme value is null. If not, return tre currentTheme
          \see IconTheme::setCurrentTheme();
          \par Implementation Note
          This function is non standard, it search for the current user desktop (gnome/kde only for now), and read
          the configuration files for the theme. The Freedesktop spec does not mention where to get this information for now.
          \par Algorithm
          \li Get the desktop loaded: environnement variable "DESKTOP_SESSION".
          \li If the desktop is "gnome", read the xml file "$HOME/.gconf/desktop/gnome/interface/%gconf.xml", and get the key "icon_theme"
          \li If the desktop is "kde", read the ini file "$HOME/.kde/share/config/kdeglobals", and search fore "[Icons]"/"Theme" key.
          If not found, use the name "default.kde4" found in "/usr/share/icons/".
          If not found, use the name "default.kde" found in "/usr/share/icons/".
          \li If all failed, returns the "hicolor" theme.
          */
        static IconTheme *getCurrentTheme();

        /**
          \brief Manually set the default theme
          \exception throw FDException if the parameter is null, or the theme can't be loaded via loadTheme() function.
          */
        static void setCurrentTheme(IconTheme *theme) throw(FDException);

        /*!
          \brief Find a theme by name
          \param name The name of the theme to find (internal name)
          \return NULL if there is no theme of that name, or an instance of the theme.
          \note The caller must not delete the return instance by himself. It is done internaly.
          */
        static IconTheme *findTheme(const std::string &name);

        /**
          \brief Fetch all available themes, and return a pointer to the list. The list contains themes internal names.

          \note The user must NOT delete this pointer. It is done internaly. Or use the cleanCache(); method instead.
          */
//        static const stringList *lookupThemes();

        /**
          \brief load the theme informations, only if not already done.
          \param Note:
          This method is called by the findIcon* methods, so it's optional to call it.
          */
        void loadTheme() throw(FDException);

        /*!
          \brief Optionnal function to clean the cache.
          \note Call of this function is not recommended. The cache will be cleanup when the program exits.
          */
        static void cleanCache();

        /* Theme informations */

        /**
          \return true if the theme as been loaded. \see loadTheme();
          */
        bool isThemeLoaded() const { return this->isLoaded; }

        /**
          \return the theme name, the value is translated if available in the theme, to the current language (UTF-8 format)
          */
        const std::string &getName() const { return this->userName; }
        /**
          \return the theme name (the name of the theme directory)
          */
        const std::string &getInternalName() const { return this->internalName; }

        /**
          \return the theme comment, in locale language if available (UTF-8 format)
          */
        const std::string &getComment() const { return this->comment; }

        /**
          \return true, if this theme should not be displayed to the user
          */
        bool isHidden() const { return this->hidden; }

        /**
          \return the icon name for presening this theme
          */
        const std::string &getExample() const { return this->example; }

        /**
          \return a list of themes names which this theme inherits.
          */
        const stringList &getInheritsThemesNames() const { return this->inherits; }

        /**
          \return the list of sub icons directories of this theme.
          */
        const stringList &getDirectoriesNames() const { return this->directories; }

        /* Publics freedesktop's icons lookup fucntions */

        /**
          \brief Find an Icon for the current theme.
          \param[in] icon The icon name (a string like found in IconNaming classes
          \param[in] size The desired icon size (ex: "32" for "32x32").
          \exception FDException Can throw an FDException if somethings goes wrong (like the default theme is NULL).
          \return the full path of this icon, or an empty string if not found.
          */
        static std::string findIcon(const std::string &icon, int size) throw(FDException);
        
        /**
          \brief Find an icon for a specific theme.
          \param[in] icon  The icon name (a string like found in IconNaming classes
          \param[in] size The desired icon size (ex: "32" for "32x32").
          \param[in] theme The theme of icon desired.
          \exception FDException Can throw an FDException if somethings goes wrong (like the theme param is NULL).
          \return the full path of this icon, or an empty string if not found.
          */
        static std::string findIconHelper(const std::string &icon, int size, IconTheme *theme) throw(FDException);

        /**
          \brief Find an icon for theses names.
          \param[in] iconList a string list which contains a list of icons name
          \param[in] size The desired icon size (ex: "32" for "32x32").
          \exception FDException Can throw an FDException if somethings goes wrong (like the theme param is NULL).
          \par Explanation
            In some cases you don't always want to fall back to an icon in an inherited theme.
            For instance, sometimes you look for a set of icons, prefering any of them before
            using an icon from an inherited theme. To support such operations implementations
            can contain a function that finds the first of a list of icon names in the inheritance hierarchy.
          */
        static std::string findBestIcon(stringList &iconList, int size) throw (FDException);

        /**
          \brief Find an icon for theses names.
          \param[in] iconList a string list which contains a list of icons name
          \param[in] size The desired icon size (ex: "32" for "32x32").
          \param[in] theme The theme of icon desired.
          \exception FDException Can throw an FDException if somethings goes wrong (like the theme param is NULL).
          \par Explanation
            In some cases you don't always want to fall back to an icon in an inherited theme.
            For instance, sometimes you look for a set of icons, prefering any of them before
            using an icon from an inherited theme. To support such operations implementations
            can contain a function that finds the first of a list of icon names in the inheritance hierarchy.
          */
        static std::string findBestIconHelper(stringList &iconList, int size, IconTheme *theme) throw (FDException);

    private:
        /**
          \brief Private constructor, for cache auto cleanup.
          */
        IconTheme();

        static void initThemesList();

        const std::string *getLocaleValue(const IniKeysValues *themeSection, const std::string *_key) const;

	/**
	 * Try to load the gnome theme
	 */
	static IconTheme *getGnomeTheme();

	/**
	 * Build a theme, which inherits all themes found in "/usr/share/icons".
	 */
	static void buildVirtualTheme();

        /**
          Check for problems occurs in the directories list in the "Icon Theme" section and
          the directories provided by directories sections.
          */
        void checkThemeProblems();

        /* Privates freedesktop's icons lookup fucntions */

        /**
          \brief LookupIcon (iconname, size, theme)
          */
       static std::string lookupIcon(const std::string &iconname, int size, IconTheme *theme);

       /**
         \brief LookupFallbackIcon (iconname)
         */
       static std::string lookupFallbackIcon(const std::string &iconname);

        /* Class (static) properties */

        /*!
          \brief The subdirectory "icons"
         */
        static const std::string ICONS_THEMES_SPEC_SUBDIRECTORY;

        /*!
            \brief Icons and themes are looked for in a set of directories.
            \details By default, apps should look in $HOME/.icons (for backwards compatibility),
                in $XDG_DATA_DIRS/icons and in /usr/share/pixmaps (in that order). Applications may further add their own icon directories to this list,
                and users may extend or change the list (in application/desktop specific ways).In each of these directories themes are stored as subdirectories.
                A theme can be spread across several base directories by having subdirectories of the same name. This way users can extend and override system themes.

            \note I found icons in XDG_DATA_HOME/icons too.
         */
        static const std::string &GetIconsThemesSpecDirectories();

        /*!
          In at least one of the theme directories there must be a file called index.theme that describes the theme.
            The first index.theme found while searching the base directories in order is used. This file describes the general attributes of the theme.
          */
        static const std::string THEME_FILE;

        /*!
            \brief In order to have a place for third party applications to install their icons there should always exist a theme called "hicolor".
            \details The data for the hicolor theme is available for download at: http://www.freedesktop.org/software/icon-theme/.
                Implementations are required to look in the "hicolor" theme if an icon was not found in the current theme.
            \note This name is chosen for backwards compatibility with the old KDE default theme
          */
        static const std::string DEFAULT_THEME;


        /*!
          \brief Icon's themes directories of the current system (fetch using XDGBaseDirectory class).
          */
        static const stringList &GetIconsThemesDirectories();

        /*!
          \brief An icon file is an image that can be loaded and used as an icon.
          \details The supported image file formats are PNG, XPM and SVG. PNG is the recommended bitmap format, and SVG is for vectorized icons.
          XPM is supported due to backwards compability reasons, and it is not recommended that new themes use XPM files. Support for SVGs is optional.
          */
        static int ICONS_FILES_FORMATS;

        /**
          List of icons extensions supported.
          */
        static stringList *iconsExtensions;

        /**
          \brief The current user theme.
          */
        static IconTheme *currentTheme;

        /* Instance properties */

        /*!
          \internal For internal use only. Deallocate static pointer ref.
          */
        bool flagStaticInstance;

        /*!
          \internal For internal use only. Deallocate static pointer ref.
          */
        static IconTheme staticInstance;

        /**
          List of themes already loaded
          */
        static HMapThemes *themesList;

        /**
          Directories of the theme.
          */
        stringList baseDirectories;

        /**
          The theme full filename (directory + index.theme)
          */
        std::string themeFilename;

        /**
          Flag indicate that this theme is loaded.
          */
        bool isLoaded;

        /**
          \brief The internal name of the theme is the name of the subdirectory.
          */
        std::string internalName;

        /**
          \brief The user-visible name (the local name if it exists, or the default name).
          \note
            Theme names are case sensitive, and are limited to ASCII characters. Theme names may also not contain comma or space.
          */
        std::string userName;

        /**
          \brief The comment of this theme, locale text if available.
          \par longer string describing the theme
          */
        std::string comment;

        /**
          \brief List of inherits themes
          \par The name of the theme that this theme inherits from. If an icon name is not found in the
            current theme, it is searched for in the inherited theme (and recursively in all the inherited themes).
            If no theme is specified implementations are required to add the "hicolor" theme to the inheritance
            tree. An implementation may optionally add other default themes in between the last specified
            theme and the hicolor theme.
          */
        stringList inherits;

        /**
          \brief List of sub themes icons directories.
          \par list of subdirectories for this theme. For every subdirectory there must be a section in the
            index.theme file describing that directory.
          */
        stringList directories;

        /**
          \brief Hidden theme ?
          \par Whether to hide the theme in a theme selection user interface.
            This is used for things such as fallback-themes that are not supposed to be visible to the user.
          */
        bool hidden;

        /**
          \brief The example string.
          \par The name of an icon that should be used as an example of how this theme looks.
          */
        std::string example;

        /**
          The content of directories sections.
          */
        ListThemeDirectories *listThemeDirectories;
    };
}

#endif // ICONTHEME_H
