/*
    Copyright (C) 2010 Yvon Tanguy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include<stdlib.h>
#include<sys/stat.h>
#include<sys/types.h>
#include<errno.h>
#include<string.h>
#ifdef __unix__
#include<wordexp.h>
#include<pwd.h>
#endif
#ifdef __MINGW32__
#include<io.h>
#endif
#include "libfreedesktop.h"
#include "xdgbasedirectory.h"

using namespace std;

using namespace freedesktop;

/* Constantes definitions */
/*
const std::string XDGBaseDirectory::ENV_XDG_DATA_DIRS = "XDG_DATA_DIRS";
const std::string XDGBaseDirectory::ENV_XDG_CONFIG_DIRS = "XDG_CONFIG_DIRS";
const std::string XDGBaseDirectory::ENV_XDG_DATA_HOME = "XDG_DATA_HOME";
const std::string XDGBaseDirectory::ENV_XDG_CONFIG_HOME = "XDG_CONFIG_HOME";
const std::string XDGBaseDirectory::ENV_XDG_CACHE_HOME = "XDG_CACHE_HOME";
const std::string XDGBaseDirectory::ENV_XDG_RUNTIME_DIR = "XDG_RUNTIME_DIR";
const std::string XDGBaseDirectory::ENV_SYSCONF_DIR = "sysconfdir";
*/

#ifdef __unix__
#define __DIRECTORY_SEPARATOR_CHAR ":"
const std::string XDGBaseDirectory::DEFAULT_XDG_DATA_DIRS = "/usr/local/share/:/usr/share/";
const std::string XDGBaseDirectory::DEFAULT_XDG_CONFIG_DIRS = "$sysconfdir/xdg";
const std::string XDGBaseDirectory::DEFAULT_XDG_DATA_HOME = "${HOME}/.local/share";
const std::string XDGBaseDirectory::DEFAULT_XDG_CONFIG_HOME = "${HOME}/.config";
const std::string XDGBaseDirectory::DEFAULT_XDG_CACHE_HOME = "${HOME}/.cache";
const std::string XDGBaseDirectory::DEFAULT_XDG_RUNTIME_DIR = "/tmp/${USER}-runtime"; // FIXME not from the spec, comes from my own: See mailing list if something has been define.
const std::string XDGBaseDirectory::DEFAULT_HOME_DIR = "$HOME";
const std::string XDGBaseDirectory::DEFAULT_SYSCONF_DIR = "/etc";
#elif WIN32
// experimental !
#define __DIRECTORY_SEPARATOR_CHAR ";"
const std::string XDGBaseDirectory::DEFAULT_HOME_DIR = "%HOMEDRIVE%%HOMEPATH%";
const std::string XDGBaseDirectory::DEFAULT_XDG_DATA_DIRS = "%CommonProgramFiles%\\share";
const std::string XDGBaseDirectory::DEFAULT_XDG_CONFIG_DIRS = "%CommonProgramFiles%\\xdg";
const std::string XDGBaseDirectory::DEFAULT_XDG_DATA_HOME = "%APPDATA%";
const std::string XDGBaseDirectory::DEFAULT_XDG_CONFIG_HOME = "%APPDATA%\\config";
const std::string XDGBaseDirectory::DEFAULT_XDG_CACHE_HOME = "%APPDATA%\\cache";
const std::string XDGBaseDirectory::DEFAULT_XDG_RUNTIME_DIR = "%Temp%";
const std::string XDGBaseDirectory::DEFAULT_SYSCONF_DIR = "%CommonProgramFiles%";
#endif

stringList XDGBaseDirectory::xdg_data_dirs; // = stringList();
stringList XDGBaseDirectory::xdg_config_dirs; // = stringList();
std::string XDGBaseDirectory::xdg_data_home = EMPTY_STRING;
std::string XDGBaseDirectory::xdg_config_home = EMPTY_STRING;
std::string XDGBaseDirectory::xdg_cache_home = EMPTY_STRING;
std::string XDGBaseDirectory::home_dir = EMPTY_STRING;
std::string XDGBaseDirectory::xdg_runtime_dir = EMPTY_STRING;
std::string XDGBaseDirectory::sysconf_dir = EMPTY_STRING;

XDGBaseDirectory::XDGBaseDirectory() {
}

stringList XDGBaseDirectory::GetXDGDirs(const string &xdg_dirs, const string &subdirname) throw(FDException) {
    stringList listXDGDirs;
    listXDGDirs.empty();

    // Test input parameters first!
    if (!xdg_dirs.empty() && !subdirname.empty()) {
        stringList xdg_string_array;
        stringSplit(xdg_dirs, xdg_string_array, GetDirectorySeparatorChar());

        for (size_t i=0; i<xdg_string_array.size(); i++) {
            string xdg_dir = xdg_string_array[i];
            if (GetEnvXDGDataDirs() == xdg_dir) {
                GetDataDirs();
                for (size_t j=0; j<xdg_data_dirs.size(); j++) {
                    string tmp_dir = xdg_data_dirs[j] + DIR_SEPARATOR_CHAR + subdirname;
                    stringReplace(tmp_dir, DOUBLE_DIR_SEPARATOR_CHAR, DIR_SEPARATOR_CHAR);
                    listXDGDirs.push_back(tmp_dir);
                }
            } else if (GetEnvXDGConfigDirs() == xdg_dir) {
                GetConfigDirs();
                for (size_t j=0; j<xdg_config_dirs.size(); j++) {
                    string tmp_dir = xdg_config_dirs[j] + DIR_SEPARATOR_CHAR + subdirname;
                    stringReplace(tmp_dir, DOUBLE_DIR_SEPARATOR_CHAR, DIR_SEPARATOR_CHAR);
                    listXDGDirs.push_back(tmp_dir);
                }
            } else if (GetEnvXDGDataHome() == xdg_dir) {
                GetDataHome();
                string tmp_dir = xdg_data_home + DIR_SEPARATOR_CHAR + subdirname;
                stringReplace(tmp_dir, DOUBLE_DIR_SEPARATOR_CHAR, DIR_SEPARATOR_CHAR);
                listXDGDirs.push_back(tmp_dir);
            } else if (GetEnvXDGConfigHome() == xdg_dir) {
                GetConfigHome();
                string tmp_dir = xdg_config_home + DIR_SEPARATOR_CHAR + subdirname;
                stringReplace(tmp_dir, DOUBLE_DIR_SEPARATOR_CHAR, DIR_SEPARATOR_CHAR);
                listXDGDirs.push_back(tmp_dir);
            } else if (GetEnvXDGCacheHome() == xdg_dir) {
                GetCacheHome();
                string tmp_dir = xdg_cache_home + DIR_SEPARATOR_CHAR + subdirname;
                stringReplace(tmp_dir, DOUBLE_DIR_SEPARATOR_CHAR, DIR_SEPARATOR_CHAR);
                listXDGDirs.push_back(tmp_dir);
            } else if (GetEnvXDGRuntimeDir() == xdg_dir) {
                try {
                    GetRuntimeDir();
                } catch (FDException e) {
                    e.addTrace(__FILE__, __LINE__, __func__);
                    throw e;
                }

                string tmp_dir = xdg_runtime_dir + DIR_SEPARATOR_CHAR + subdirname;
                stringReplace(tmp_dir, DOUBLE_DIR_SEPARATOR_CHAR, DIR_SEPARATOR_CHAR);
                listXDGDirs.push_back(tmp_dir);
            } else {
                string tmp_dir = EnvPathToRealPath(xdg_dir);
                listXDGDirs.push_back(tmp_dir);
            }
        }
    }

    return listXDGDirs;
}

const std::string &XDGBaseDirectory::GetSysConfDir() {
    if (sysconf_dir.empty()) {
        char *envVal = getenv(GetEnvSysconfDir().c_str());
        if (NULL == envVal) {
            sysconf_dir = DEFAULT_SYSCONF_DIR;
#ifdef HAVE_SETENV
            setenv(GetEnvSysconfDir().c_str(), DEFAULT_SYSCONF_DIR.c_str(), 0);
#else
            putenv((char *) (GetEnvSysconfDir() + "=" + DEFAULT_SYSCONF_DIR).c_str());
#endif
        } else {
            sysconf_dir = envVal;
        }
    }

    return sysconf_dir;
}

const stringList &XDGBaseDirectory::GetDataDirs() {
    if (xdg_data_dirs.empty()) {
        string var_data_dirs;
        char *envValue = getenv(GetEnvXDGDataDirs().c_str());
        if (NULL != envValue) {
            var_data_dirs = envValue;
        }
        // use default if empty
        if (var_data_dirs.empty()) {
            var_data_dirs = DEFAULT_XDG_DATA_DIRS;
        }

        stringList tmp_data_dirs;
        stringSplit(var_data_dirs, tmp_data_dirs, GetDirectorySeparatorChar());

        for (size_t i=0; i<tmp_data_dirs.size(); i++) {
            xdg_data_dirs.push_back(EnvPathToRealPath(tmp_data_dirs[i]));
        }

    }

    return xdg_data_dirs;
}

const stringList &XDGBaseDirectory::GetConfigDirs() {
    if (xdg_config_dirs.empty()) {
        char *envVal = getenv(GetEnvXDGConfigDirs().c_str());
        string var_config_dirs = EMPTY_STRING;
        if (NULL != envVal) {
          var_config_dirs = envVal;
        }

        // use default if empty
        if (var_config_dirs.empty()) {
            (void) GetSysConfDir();
            var_config_dirs = DEFAULT_XDG_CONFIG_DIRS;
        }

        stringList tmp_config_dirs;
        stringSplit(var_config_dirs, tmp_config_dirs, GetDirectorySeparatorChar());

        for (size_t i=0; i<tmp_config_dirs.size(); i++) {
            xdg_config_dirs.push_back(EnvPathToRealPath(tmp_config_dirs[i]));
        }

    }

    return xdg_config_dirs;
}

const string &XDGBaseDirectory::GetDataHome() {
    if (xdg_data_home.empty()) {
        char *envVal = getenv(GetEnvXDGDataHome().c_str());
        string tmp_data_home = EMPTY_STRING;
        if (NULL != envVal) {
            tmp_data_home = envVal;
        }
        tmp_data_home = EnvPathToRealPath(tmp_data_home);

        if (tmp_data_home.empty()) {
            xdg_data_home = EnvPathToRealPath(DEFAULT_XDG_DATA_HOME);
        } else {
            xdg_data_home = EnvPathToRealPath(tmp_data_home);
        }
    }
    return xdg_data_home;
}

const string &XDGBaseDirectory::GetConfigHome() {
    if (xdg_config_home.empty()) {
        char *envVal = getenv(GetEnvXDGConfigHome().c_str());
        string tmp_config_home = EMPTY_STRING;
        if (NULL != envVal) {
            tmp_config_home = envVal;
        }
        tmp_config_home = EnvPathToRealPath(tmp_config_home);

        if (tmp_config_home.empty()) {
            xdg_config_home = EnvPathToRealPath(DEFAULT_XDG_CONFIG_HOME);
        } else {
            xdg_config_home = EnvPathToRealPath(tmp_config_home);
        }
    }
    return xdg_config_home;
}

const string &XDGBaseDirectory::GetCacheHome() {
    if (xdg_cache_home.empty()) {
        char *envVal = getenv(GetEnvXDGCacheHome().c_str());
        string tmp_cache_home = EMPTY_STRING;
        if (NULL != envVal) {
            tmp_cache_home = envVal;
        }
        tmp_cache_home = EnvPathToRealPath(tmp_cache_home);

        if (tmp_cache_home.empty()) {
            xdg_cache_home = EnvPathToRealPath(DEFAULT_XDG_CACHE_HOME);
        } else {
            xdg_cache_home = EnvPathToRealPath(tmp_cache_home);
        }
    }

    return xdg_cache_home;
}

const string &XDGBaseDirectory::GetRuntimeDir() throw (FDException) {
    char *envValue = NULL;
    if (xdg_runtime_dir.empty()) {
        envValue = getenv(GetEnvXDGRuntimeDir().c_str());
        string tmp_runtime_dir = EMPTY_STRING;
        if (NULL != envValue) {
          tmp_runtime_dir = envValue;
        }
        if (tmp_runtime_dir.empty()) {
            xdg_runtime_dir = EnvPathToRealPath(DEFAULT_XDG_RUNTIME_DIR);
        } else {
            xdg_runtime_dir = EnvPathToRealPath(tmp_runtime_dir);
        }
//#ifdef HAVE_MKDIR
        // check if the directoty exists, by trying to created it (perm 700)
#ifdef __MINGW32__
        if (mkdir(xdg_runtime_dir.c_str())) {
#else
        if (mkdir(xdg_runtime_dir.c_str(), S_IRWXU)) {
#endif

            int errNo = errno;
            if (errNo != EEXIST) {
                // here we do not know what to do.
                xdg_runtime_dir = EMPTY_STRING;
                char *strError = strerror(errNo);
                string errorMessage = "XDG_RUNTIME_DIR not available: ";
                errorMessage += strError;
                throw FD_EXCEPTION(errorMessage);
            }
        }
//#endif
    }
    return xdg_runtime_dir;
}

const string &XDGBaseDirectory::GetHomeDir() {
#ifdef __unix__
    struct passwd *pw;

    if (home_dir.empty()) {

      // Valgrind: memleak here (TODO check why):
      /*
==4161== HEAP SUMMARY:
==4161==     in use at exit: 300 bytes in 11 blocks
==4161==   total heap usage: 1,559 allocs, 1,548 frees, 98,411 bytes allocated
==4161== 
==4161== 300 (60 direct, 240 indirect) bytes in 1 blocks are definitely lost in loss record 11 of 11
==4161==    at 0x4C274A8: malloc (vg_replace_malloc.c:236)
==4161==    by 0x56D57FA: nss_parse_service_list (nsswitch.c:622)
==4161==    by 0x56D611D: __nss_database_lookup (nsswitch.c:164)
==4161==    by 0x5D603CF: ???
==4161==    by 0x5D60E5C: ???
==4161==    by 0x56D67F2: __nss_setent (getnssent_r.c:84)
==4161==    by 0x56842CB: setpwent (getXXent_r.c:127)
==4161==    by 0x4037E1: freedesktop::XDGBaseDirectory::GetHomeDir() (in libfreedesktop/tests/test_xdgbasedirectory)
==4161==    by 0x402027: GetHomeDirTest() (in libfreedesktop/tests/test_xdgbasedirectory)
==4161==    by 0x402109: main (in libfreedesktop/tests/test_xdgbasedirectory)
==4161== 
==4161== LEAK SUMMARY:
==4161==    definitely lost: 60 bytes in 1 blocks
==4161==    indirectly lost: 240 bytes in 10 blocks
==4161==      possibly lost: 0 bytes in 0 blocks
==4161==    still reachable: 0 bytes in 0 blocks
==4161==         suppressed: 0 bytes in 0 blocks
       */

        setpwent();
        pw = getpwuid(getuid());
        
        if (pw && pw->pw_dir) {
            home_dir = pw->pw_dir;
        } else {
            home_dir = EnvPathToRealPath(DEFAULT_HOME_DIR);
        }

	endpwent();
    }
#else
    if (home_dir.empty()) {
        home_dir = EnvPathToRealPath(DEFAULT_HOME_DIR);
    }
#endif
    return home_dir;
}

string XDGBaseDirectory::EnvPathToRealPath(const string &path) {
    string realPath;

#ifdef __unix__
    wordexp_t wep;
    char **output;

    // Test input parameter first!
    if (!path.empty()) {
        const char *cstr_path = path.c_str();
        wordexp(cstr_path, &wep, 0);
        output = wep.we_wordv;
        if (wep.we_wordc == 1) {
            realPath.assign(output[0]);
        }
        wordfree(&wep);
    }
#elif WIN32
    if (!path.empty()) {
        realPath = path;
        string::size_type start = realPath.find("%", 0);
        // FIXME: No '%' possible in a path!
        // FIXME buggy function
        while (start != string::npos) {
            int envStart = 0;
            int envEnd = 0;

            envStart = realPath.find("%", start);
            envEnd = realPath.find('%', start+1);

            if (envStart != -1 && envEnd != -1) {

                string envName = realPath.substr(envStart+1, envEnd-envStart-1);
                char * envValPtr = getenv(envName.c_str());
                string envVal = "";
                if (envValPtr != NULL) {
                    envVal = envValPtr;
                }

                realPath.replace(realPath.begin()+envStart, realPath.begin()+envEnd+1, envVal);

                start = realPath.find("%", 0);
            } else {
                start = string::npos;
            }
        }
    }
#endif
    return realPath;
}

const std::string &XDGBaseDirectory::GetEnvXDGDataDirs() {
    static const std::string ENV_XDG_DATA_DIRS = "XDG_DATA_DIRS";

    return ENV_XDG_DATA_DIRS;
}

const std::string &XDGBaseDirectory::GetEnvXDGConfigDirs() {
    static const std::string ENV_XDG_CONFIG_DIRS = "XDG_CONFIG_DIRS";

    return ENV_XDG_CONFIG_DIRS;
}

const std::string &XDGBaseDirectory::GetEnvXDGDataHome() {
    static const std::string ENV_XDG_DATA_HOME = "XDG_DATA_HOME";

    return ENV_XDG_DATA_HOME;
}

const std::string &XDGBaseDirectory::GetEnvXDGConfigHome() {
    static const std::string ENV_XDG_CONFIG_HOME = "XDG_CONFIG_HOME";

    return ENV_XDG_CONFIG_HOME;
}

const std::string &XDGBaseDirectory::GetEnvXDGCacheHome() {
    static const std::string ENV_XDG_CACHE_HOME = "XDG_CACHE_HOME";

    return ENV_XDG_CACHE_HOME;
}

const std::string &XDGBaseDirectory::GetEnvXDGRuntimeDir() {
    static const std::string ENV_XDG_RUNTIME_DIR = "XDG_RUNTIME_DIR";

    return ENV_XDG_RUNTIME_DIR;
}

const std::string &XDGBaseDirectory::GetEnvSysconfDir() {
    static const std::string ENV_SYSCONF_DIR = "sysconfdir";

    return ENV_SYSCONF_DIR;
}

const std::string &XDGBaseDirectory::GetDirectorySeparatorChar() {
    static const std::string DirectorySeparatorChar = __DIRECTORY_SEPARATOR_CHAR;

    return DirectorySeparatorChar;
}
