/*
    Copyright (C) 2010 Yvon Tanguy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef XDGBASEDIRECTORY_H
#define XDGBASEDIRECTORY_H

#include<stdlib.h>
#include<sys/types.h>
#ifdef __unix__
#include<wordexp.h>
#include<pwd.h>
#endif
#include "libfreedesktop.h"
#include "fdexception.h"

namespace freedesktop {

    /*!
        \brief Implementation of "XDG Base Directory Specification"
        \par Concepts
         The XDG Base Directory Specification is based on the following concepts:
         - There is a single base directory relative to which user-specific data files should be written.
           This directory is defined by the environment variable $XDG_DATA_HOME.
         - There is a single base directory relative to which user-specific configuration files should be written.
           This directory is defined by the environment variable $XDG_CONFIG_HOME.
         - There is a set of preference ordered base directories relative to which data files should be searched.
           This set of directories is defined by the environment variable $XDG_DATA_DIRS.
         - There is a set of preference ordered base directories relative to which configuration files
           should be searched. This set of directories is defined by the environment variable $XDG_CONFIG_DIRS.
         - There is a single base directory relative to which user-specific non-essential (cached) data
           should be written. This directory is defined by the environment variable $XDG_CACHE_HOME.
        \par Source:
            <a href="http://standards.freedesktop.org/basedir-spec/basedir-spec-0.7.html">XDG Base Directory Specification v0.7</a>
    */
    class XDGBaseDirectory {

    private:
/*
        static const std::string ENV_XDG_DATA_DIRS;
        static const std::string ENV_XDG_CONFIG_DIRS;
        static const std::string ENV_XDG_DATA_HOME;
        static const std::string ENV_XDG_CONFIG_HOME;
        static const std::string ENV_XDG_CACHE_HOME;
        static const std::string ENV_XDG_RUNTIME_DIR;
        static const std::string ENV_SYSCONF_DIR;

        static const std::string DIRECTORY_SEPARATOR_CHAR;
*/
        static const std::string DEFAULT_XDG_DATA_DIRS;
        static const std::string DEFAULT_XDG_CONFIG_DIRS;
        static const std::string DEFAULT_XDG_DATA_HOME;
        static const std::string DEFAULT_XDG_CONFIG_HOME;
        static const std::string DEFAULT_XDG_CACHE_HOME;
        static const std::string DEFAULT_XDG_RUNTIME_DIR;
        static const std::string DEFAULT_HOME_DIR;
        static const std::string DEFAULT_SYSCONF_DIR;

    private:
        // because of this is static class, prevent instanciations:
        XDGBaseDirectory();

    private:
        // variables to store values fetched
        static stringList xdg_data_dirs;
        static stringList xdg_config_dirs;
        static std::string xdg_data_home;
        static std::string xdg_config_home;
        static std::string xdg_cache_home;
        static std::string xdg_runtime_dir;
        static std::string home_dir;
        static std::string sysconf_dir;

    public:

        /*!
          \brief This helper function, return a string array, with xdg dirs, depending of xdg_dirs and subdirname.
          \param xdg_dirs   A string containing ENV_* string, separated by a coma (':')
          \param subdirname The directory to append to alls xdg dirs. It will be append only for ENV_XDG* directories.
                 Instead, the environnement variables will be transform to real path, using EnvPathToRealPath() function.
          \return return an xdg dir array (stringList), or an empty array if one parameter is not set.
         */
        static stringList GetXDGDirs(const std::string &xdg_dirs, const std::string &subdirname) throw(FDException);

        /**
          \brief Default configuration files should be installed to $sysconfdir/xdg/subdir/filename with $sysconfdir defaulting to /etc.
          */
        static const std::string &GetSysConfDir();

        /*!
          \brief $XDG_DATA_DIRS defines the preference-ordered set of base directories to search for data files
          in addition to the $XDG_DATA_HOME base directory.
          The directories in $XDG_DATA_DIRS should be seperated with a colon ':'.
          This function does not include XDG_DATA_HOME dir
          \return a stringList of $XDG_DATA_DIRS directories, for the current system
         */
        static const stringList &GetDataDirs();

        /*!
          \brief $XDG_CONFIG_DIRS defines the preference-ordered set of base directories to search for configuration
            files in addition to the $XDG_CONFIG_HOME base directory.
            The directories in $XDG_CONFIG_DIRS should be seperated with a colon ':'.
            If $XDG_CONFIG_DIRS is either not set or empty, a value equal to /etc/xdg should be used.
          \return a stringList of $XDG_CONFIG_DIRS:$XDG_CONFIG_HOME directories, for the current system
         */
        static const stringList &GetConfigDirs();

        /*!
          \brief $XDG_DATA_HOME defines the base directory relative to which user specific data files should be stored.
          If $XDG_DATA_HOME is either not set or empty, a default equal to $HOME/.local/share should be used.
          \return The XDG_DATA_HOME of this system
         */
        static const std::string &GetDataHome();

        /*!
          \brief $XDG_CONFIG_HOME defines the base directory relative to which user specific configuration files
          should be stored. If $XDG_CONFIG_HOME is either not sex3deditor32t or empty, a default equal to $HOME/.config should be used.
          \return The XDG_CONFIG_HOME of this system
         */
        static const std::string &GetConfigHome();

        /*!
          \brief $XDG_CACHE_HOME defines the base directory relative to which user specific non-essential data files
          should be stored. If $XDG_CACHE_HOME is either not set or empty, a default equal to $HOME/.cache
          should be used.
          \return The XDG_CACHE_HOME of this system.
         */
        static const std::string &GetCacheHome();

        /**
          \brief $XDG_RUNTIME_DIR defines the base directory relative to which user-specific non-essential runtime
            files and other file objects (such as sockets, named pipes, ...) should be stored.
            The directory MUST be owned by the user, and he MUST be the only one having read and write access to it.
            Its Unix access mode MUST be 0700.
           \par Particularities
            The lifetime of the directory MUST be bound to the user being logged in.
            It MUST be created when the user first logs in and if the user fully logs out the directory MUST be removed.
            If the user logs in more than once he should get pointed to the same directory, and it is mandatory that the
            directory continues to exist from his first login to his last logout on the system, and not removed in between.
            Files in the directory MUST not survive reboot or a full logout/login cycle.

            The directory MUST be on a local file system and not shared with any other system. The directory MUST by fully-featured
            by the standards of the operating system. More specifically, on Unix-like operating systems AF_UNIX sockets, symbolic links,
            hard links, proper permissions, file locking, sparse files, memory mapping, file change notifications, a reliable hard link
            count must be supported, and no restrictions on the file name character set should be imposed. Files in this directory MAY
            be subjected to periodic clean-up. To ensure that your files are not removed, they should have their access time timestamp
            modified at least once every 6 hours of monotonic time or the 'sticky' bit should be set on the file.

            If $XDG_RUNTIME_DIR is not set applications should fall back to a replacement directory with similar capabilities and print
            a warning message. Applications should use this directory for communication and synchronization purposes and should not place
            larger files in it, since it might reside in runtime memory and cannot necessarily be swapped out to disk.
           \par Implementation note:
            The freedesktop spec does not mention any fallback place for this directory. So by default I use "/tmp/${USER}-runtime"
           \return The XDG_RUNTIME_DIR of this system.
           \exception FDException: If the XDG_RUNTIME_DIR does not exists, and it's creation failed.
          */
        static const std::string &GetRuntimeDir() throw (FDException);

        /*!
          \brief return the home directory of the current user ($HOME)
         */
        static const std::string &GetHomeDir();

        /*!
          \brief
          Convert a path containing environnement variable to full path.
          $HOME/.config  -> /home/me/.config
         */
        static std::string EnvPathToRealPath(const std::string &path);

        /* To prevent the "static initialization order fiasco" */
        static const std::string &GetEnvXDGDataDirs();
        static const std::string &GetEnvXDGConfigDirs();
        static const std::string &GetEnvXDGDataHome();
        static const std::string &GetEnvXDGConfigHome();
        static const std::string &GetEnvXDGCacheHome();
        static const std::string &GetEnvXDGRuntimeDir();
        static const std::string &GetEnvSysconfDir();
        static const std::string &GetDirectorySeparatorChar();
    };


}

#endif // XDGBASEDIRECTORY_H
