/*
    Copyright (C) 2010 Yvon Tanguy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef XDGUSERDIRECTORY_H
#define XDGUSERDIRECTORY_H

#include "xdgbasedirectory.h"

namespace freedesktop {

    /*!
      \brief Because common user directory names cannot be hard coded due to the localization issue,
        the desktop must maintain a kind of cross table to map the logical directory name to the real one.
        The difficulty with that is to maintain the consistency of the cross table. Indeed, the user can
        decide to delete or rename a special folder, for example, he can rename the folder Music to My music.
        In this case, the cross table must be updated with the new name.

        <a href="http://www.freedesktop.org/wiki/Specifications/common-user-directories-naming-spec">common-user-directories-naming-spec</a>
     */
    enum UserDirectories {
        XDG_DESKTOP_DIR,
        XDG_DOWNLOAD_DIR,
        XDG_TEMPLATES_DIR,
        XDG_PUBLICSHARE_DIR,
        XDG_DOCUMENTS_DIR,
        XDG_MUSIC_DIR,
        XDG_PICTURES_DIR,
        XDG_VIDEOS_DIR
    };

    /*!
      \brief common-user-directories-naming-spec
      \par Abstract
        Desktop users might be able to store their data in special directories that have the same kind of content.
        This is often the case for simple Documents but also for Music, Movies or Pictures.
        Desktop applications must be able to know what is the user storage directory for the type of data they manipulate.
        In this way, a text document will be proposed to be saved by default to the user Documents directory, and the user
        favorite music player will be able to scan the user Music directory without any user configuration.

        <a href="http://www.freedesktop.org/wiki/Specifications/common-user-directories-naming-spec">common-user-directories-naming-spec</a>
     */
    class XDGUserDirectory
    {
    public:
        /*!
          \brief Fetch for users commons directories, and then return the user directory corresponding to the "directory" argument.
          \param directory: Should be one of the enum UserDirectories value.
          \return The found directory. (TODO error cases)
         */
        static std::string getDirectory(UserDirectories directory);

        /*!
          \brief Load the "user-dirs.locale" file to get the user dirs locale
          \return a string like "fr-FR" depending on the current locale.
         */
        static std::string getDirectoriesLocale();

        /*!
          \todo Not implemented.
         */
        static void saveUserDirs();

    private:
        XDGUserDirectory();
        static void loadUserDirs();

        // User's dirs init at runtime
        static bool isXDGDirLoaded;
        static std::string xdgDesktopDir;
        static std::string xdgDownloadDir;
        static std::string xdgTemplatesDir;
        static std::string xdgPublicshareDir;
        static std::string xdgDocumentsDir;
        static std::string xdgMusicDir;
        static std::string xdgPicturesDir;
        static std::string xdgVideosDir;

        // User's dirs config files
        static const std::string systemFileDirsDefaults; // "user-dirs.defaults"
        static const std::string userFileDirsLocale; // "user-dirs.locale"
        static const std::string userFileDirsDirs; // "user-dirs.dirs"
    };
}
#endif // XDGUSERDIRECTORY_H
